package fips.sim.dig.base;

import java.util.ArrayList;

/** Class holding parameter description, default value and
 *  current value for any component's parameter
 */
public abstract class AbstractParameter implements Parameter {

/** Short name of this parameter
 */
  protected String		name;

  protected String		description;


/** Default value for this parameter
 */
  protected Object		defaultval;


  protected Object		currentval;



/** If true changing may alter the size of the component. This
 *  value tells the editor and the user to take care of allready
 *  made connections.
 */
  protected boolean		alterssize;

/** If true changing may alter the pinout of this component
 */
  protected boolean		alterspinout;

/** If true changing this parameter may alter the number or
 *  value of other params.
 */
  protected boolean		altersparams;

  private ArrayList		listeners;

  public AbstractParameter(String name, String description,
                           Object defaultval, Object currentval,
                           boolean alterssize, boolean alterspinout,
                           boolean altersparams) {
    this.name=name;
    this.description=description;
    this.defaultval=defaultval;
    this.currentval=currentval;
    this.alterssize=alterssize;
    this.alterspinout=alterspinout;
    this.altersparams=altersparams;

    listeners=new ArrayList();
  }

  public synchronized void addParameterChangeListener(ParameterChangeListener pcl) {
    if(pcl!=null) listeners.add(pcl);
  }

  public synchronized void removeParameterChangeListener(ParameterChangeListener pcl) {
    listeners.remove(pcl);
  }


  public String getName() { return(name); }
  public String getDescription() { return(description); }

  public Object getValue() { return(currentval); }
  public void setValue(Object o) throws IllegalArgumentException {
    if(!isValidParameter(o)) throw new IllegalArgumentException();
    synchronized(this) {
      currentval=o;
      for(int i=0; i<listeners.size(); i++) {
        ((ParameterChangeListener)(listeners.get(i))).parameterChanged(this);
      }
    }
  }

  public Object getDefaultValue() { return(defaultval); }

  public boolean altersSize() { return(alterssize); }
  public boolean altersPinout() { return(alterspinout); }
  public boolean altersParameters() { return(altersparams); }


  public abstract boolean isValidParameter(Object o);
  public abstract Object parseString(String str) throws IllegalArgumentException;

  public abstract String toString(Object o);


  public String toString() {
    return(toString(currentval));
  }

}






