package fips.sim.dig.base;


import java.awt.Graphics;


/** Interface for all components.
 *  Cloning will create a component with the same internal
 *  state and settings as the parent component but unconnected
 *  pins.
 */
public interface Component extends Cloneable {

/** Name of the component type, like Led for simple Led
 */
  public String getComponentName();

/** Get a description about function and usage of this component
 */
  public String getDescription();


/** Get description of purpose of one pin.
 */
  public String getPinDescription(int num);


/** Simulator intern name of the component. Purpose of this string
 *  is to distinguish between different instances of one component.
 *  e.g. Led1 Irq_Led, etc ...
 */
  public String getName();

  public void setName(String name);

  public int getParameterCount();
  public Parameter[] getParameters();

  public int getPinCount();
  public void setPin(int num, fips.sim.dig.comp.Pin p);
  public fips.sim.dig.comp.Pin getPin(int num);


/** Get hint for the max length of next calculation step.
 *  All pin values needed for calculation are read now
 */
  public long getNextStep();


/** Do the following calculation step. If cycles is too big,
 *  IllegalArgumentException is thrown. Component changes pin
 *  states the new states are valid for the new cycle after
 *  doing the steps
 */
  public void doStep(long cycles);


/** Reset cyclecount, params, etc...
 */
  public void reset();


  public void writeState(java.io.ObjectOutputStream oos)
                         throws java.io.IOException;
  public void readState(java.io.ObjectInputStream ois)
                        throws java.io.IOException, ClassNotFoundException;




/** Get the size of the GUI in squares of size 1<<scale
 *  This value may depend on the current orientation
 */
  public java.awt.Dimension getSize();



/*
 *  Set graphics parameters and repaint component immediatly
 *  @param scale the 2 log of number of pixels for unit dist
 */
  public void setGUIParameters(java.awt.Point offset,
                          int scale, int orient);


  public int getOrientation();

  public boolean needsUpdate();


/** Tell the component to repaint the component
 */
  public void repaint(Graphics g);

  public void update(Graphics g);


/** Notification about click at pixel x, y relative to top-left
 *  corner of component
 */
  public void doClick(int x, int y, int button);


/** Get the pin position of a component. The position is
 *  relative to the origin of the component and is affected
 *  by the rotation parameter.
 */
  public java.awt.Point getPinPos(int num);


/** Cloning will create a component with the same internal
 *  state and settings as the parent component but unconnected
 *  pins and no graphics context (invisible)
 */
  public Object clone();



}




