package fips.sim.dig.base;

import java.awt.Dimension;
import java.util.logging.Logger;


/** Parameter to hold an integer size
 */
public class SizeParameter extends AbstractParameter {

  Dimension	maxval;
  Dimension	minval;

  public SizeParameter(String name, String description,
                       Dimension defaultval, Dimension currentval, 
                       Dimension minval, Dimension maxval) {
    this(name, description, defaultval, currentval, minval, maxval,
         false, false, false);
  }

  public SizeParameter(String name, String description,
                       Dimension defaultval, Dimension currentval,
                       Dimension minval, Dimension maxval,
                       boolean alterssize, boolean alterspinout,
                       boolean altersparams) {
    super(name, description, defaultval, currentval,
          alterssize, alterspinout, altersparams);

    if(currentval==null) throw new NullPointerException();

    this.maxval=maxval;
    this.minval=minval;

    if(maxval!=null) {
      if(((defaultval!=null)&&(!isSmaller(defaultval, maxval)))||
         (!isSmaller(currentval, maxval))||
         ((minval!=null)&&(!isSmaller(minval, maxval)))) {
        throw new IllegalArgumentException();
      }
    }

    if(minval!=null) {
      if(((defaultval!=null)&&(!isBigger(defaultval, minval)))||
         (!isBigger(currentval, minval))) {
        throw new IllegalArgumentException();
      }
    }

  }


  public boolean isValidParameter(Object o) {
    if((o==null)||(!(o instanceof Dimension))) return(false);

    Dimension d=(Dimension)o;
    if((d.width<0)||(d.height<0)) return(false);

    if((minval!=null)&&(!isBigger(d, minval))) return(false);
    if((maxval!=null)&&(!isSmaller(d, maxval))) return(false);
    return(true);
  }



  public Object parseString(String str) throws IllegalArgumentException {
    Logger.getLogger("digsim.base").fine(".parseString(str=\""+str+"\").");

    int i=str.indexOf('x');
    if(i<0) throw new IllegalArgumentException();

    Dimension d=new Dimension();
    d.width=Integer.parseInt(str.substring(0, i));
    d.height=Integer.parseInt(str.substring(i+1, str.length()));
    if(!isValidParameter(d)) throw new IllegalArgumentException();

    return(d);
  }


/** Returns true if width and height of test-dimesion is
 *  smaller or equal to limit.
 */
  private boolean isSmaller(Dimension test, Dimension limit) {
    return((test.width<=limit.width)&&(test.height<=limit.height));
  }
  private boolean isBigger(Dimension test, Dimension limit) {
    return((test.width>=limit.width)&&(test.height>=limit.height));
  }


  public String toString(Object o) {
    return(((Dimension)o).width+" "+((Dimension)o).height);
  }


}






