package fips.sim.dig.base;

import java.math.BigInteger;

/** Display the values using format xxx.yyy
 */
public class TimeParameter extends AbstractParameter {

  public final static BigInteger	TIME_FS=BigInteger.ONE;
  public final static BigInteger	TIME_NS=BigInteger.valueOf(1000000);


  BigInteger		lowerlim;
  BigInteger		upperlim;

  private final static String	timepref[]={"fs", "ps", "ns", "us", "ms", "s"};

  public TimeParameter(String name, String description,
                       BigInteger defaultval, BigInteger currentval, 
                       BigInteger lowerlim, BigInteger upperlim) {
    super(name, description, defaultval, currentval, false, false, false);

    this.lowerlim=lowerlim;
    this.upperlim=upperlim;
  }


  public boolean isValidParameter(Object o) {
    if((o==null)||(!(o instanceof BigInteger))) return(false);

    BigInteger val=(BigInteger)o;
    if((lowerlim!=null)&&(val.compareTo(lowerlim)<0)) return(false);
    if((upperlim!=null)&&(val.compareTo(upperlim)>0)) return(false);

    return(true);
  }



  public Object parseString(String str) throws IllegalArgumentException {
    //TRACE (".parseString(str=\""+str+"\").");

    str=str.trim();

    int numend=0;
    int comma=-1;
    for(numend=0; numend<str.length(); numend++) {
      if((str.charAt(numend)>='0')&&(str.charAt(numend)<='9')) continue;
      if(str.charAt(numend)=='.') {
        if(comma<0) {
          comma=numend;
          continue;
        }
        throw new IllegalArgumentException("Found 2 comma points");
      }
      break;
    }

    if(numend==0) {
      throw new IllegalArgumentException("Number value missing");
    }

    if(numend==str.length()) {
      throw new IllegalArgumentException("No unit for timeparameter");
    }

    String unitstr=str.substring(numend).trim();

    int kcnt=0;		// count for kilo multiplications (*1000)
    for(kcnt=0; kcnt<timepref.length; kcnt++) {
      if(unitstr.equals(timepref[kcnt])) break;
    }
    if(kcnt==timepref.length) {
      throw new IllegalArgumentException("Illegal unit \""+unitstr+"\"");
    }

    BigInteger ret=null;
    if(comma==0) ret=BigInteger.ZERO;
    else {
      ret=new BigInteger(str.substring(0, comma<0?numend:comma));
      if(comma<0) comma=numend;
    }

    while(comma+1<numend) { // do multiplications
      if(kcnt==0) {
        throw new IllegalArgumentException("Time values < fs not supported");
      }
      ret=ret.multiply(BigInteger.valueOf(1000));

      if(comma+3<numend) { // enought numbers available
        ret=ret.add(new BigInteger(str.substring(comma+1, comma+4)));
      } else {
        ret=ret.add(new BigInteger(str.substring(comma+1, numend)+
                                   "00".substring(0, comma+4-numend)));
      }

      comma+=3;
      kcnt--;
    }

    if(kcnt>0) {
      ret=ret.multiply(BigInteger.valueOf(1000).pow(kcnt));
    }

    return(ret);
  }



// format using the timepref list
  public String toString(Object o) {
    BigInteger val=(BigInteger)o;

    if(val.signum()==0) return("0s");

    StringBuffer sbuf=new StringBuffer();
    if(val.signum()<0) sbuf.append('-');
    val=val.abs();

    BigInteger ref=BigInteger.ONE;

    int i;
    for(i=0; i<timepref.length; i++) {
      ref=ref.multiply(BigInteger.valueOf(1000));
      if(ref.compareTo(val)>0) break;
    }

    if(i==timepref.length) i--;

    ref=BigInteger.valueOf(1000).pow(i);
    sbuf.append(val.divide(ref));
    val=val.mod(ref);
    if(val.signum()!=0) {
      sbuf.append('.');
      String str=val.toString();

      int keep=str.length();
      while(str.charAt(keep-1)=='0') keep--;

      for(int j=str.length(); j<i*3; j++) sbuf.append('0');
      sbuf.append(str.substring(0, keep));
    }
    sbuf.append(timepref[i]);
    return(sbuf.toString());
  }


}






