package fips.sim.dig.comp;

import java.awt.Color;
import java.awt.Graphics;
import java.util.logging.Logger;

public class Button extends AbstractComponent {

  private Logger	logger;

  private int		state, newstate;

  private Color		border=new Color(0xC0C0C0);
  private Color		scolors[]={ Color.black, Color.red};


  public Button() {
    super(1);

    logger=Logger.getLogger("digsim.comp");

    state=newstate=Pin.LOW;
    size=new java.awt.Dimension(1, 1);

    setPinValue(state, 0);
  }


  public String getComponentName() {
    return("Button");
  }

  public String getDescription() {
    return("Small Button");
  }


  public String getPinDescription(int num) {
    if(num==0) return("State");
    else throw new IllegalArgumentException("Invalid pin number "+num);
  }

  public int getParameterCount() { return(0); }

  public fips.sim.dig.base.Parameter[] getParameters() {
    return(null);
  }





/** Get max step length
*/
  public long getNextStep() {
    return(Long.MAX_VALUE);
  }



/** Do the following calculation step. If cycles is too big,
 *  IllegalArgumentException is thrown
 */
  public void doStep(long cycles) {
    if(state!=newstate) {
      setPinValue(newstate, 0);
      state=newstate;
    }
  }




/** Reset cyclecount, params, etc...
 */
  public void reset() {
    throw new InternalError();
  }



  public void writeState(java.io.ObjectOutputStream oos)
                         throws java.io.IOException {
    oos.write((state&1)|((newstate&1)<<1));
  }
 
 
  public void readState(java.io.ObjectInputStream ois)
                        throws java.io.IOException {
    int i=ois.read();
    state=i&1;
    newstate=(i>>1)&1;
    setPinValue(state, 0);

    needsupdate=true;
  }


  public void repaint(Graphics g) {
    logger.finer(".repaint()");

    if(scale>1) {
      g.setColor(border);
      g.drawRect(offset.x, offset.y,
                        (size.width<<scale)-1, (size.height<<scale)-1);
    }
    update(g);
  }


  public void update(Graphics g) {
    logger.finer("Updating graphics of "+getName()+" for state "+newstate);

    g.setColor(scolors[newstate]);

    if(scale>1) {
      g.fillRect(offset.x+1, offset.y+1,
                        (size.width<<scale)-2, (size.height<<scale)-2);
    } else {
      g.fillRect(offset.x, offset.y,
                        (size.width<<scale), (size.height<<scale));
    }
    needsupdate=false;
  }





  public void doClick(int x, int y, int button) {
    logger.finer("Button "+getName()+" clicked");
    newstate=newstate^0x1;
    needsupdate=true;
  }



/** Get the connector position for the pin
 */
  public java.awt.Point getPinPos(int num) {
    return(new java.awt.Point(orientation>>1,
                              (orientation==0||orientation==3)?0:1));
  }




// ************************************************************************* //
//
//			Internal Methods				     //
//
// ************************************************************************* //






  public Object clone() {
    Button clone=new Button();

    clone.name=name;
    clone.state=state;
    clone.newstate=newstate;
    clone.size=(java.awt.Dimension)(size.clone());
    clone.border=border;
    clone.scolors=(Color[])(scolors.clone());
    return(clone);
  }
}



