package fips.sim.dig.comp;

import java.awt.Dimension;
import java.awt.Graphics;


import fips.sim.dig.base.Parameter;
import fips.sim.dig.base.TimeParameter;


public class Clock extends AbstractComponent {

  final static int		states[]={Pin.LOW, Pin.UNKNOWN, Pin.HIGH, Pin.UNKNOWN};
  final static int		istates[]={Pin.HIGH, Pin.UNKNOWN, Pin.LOW, Pin.UNKNOWN};


/** Current state cnt. */
  int				state;

  private final static long	defwaits[]={2000000000000000L, 500000000000000L,
                                            2000000000000000L, 500000000000000L};
  private long			waits[]=defwaits;

  long				nextchg;

  java.awt.Color		colors[]={java.awt.Color.black,
                                          java.awt.Color.red,
                                          java.awt.Color.gray};

  private final static String	paramnames[]={"Low", "Rise", "High", "Fall"};
  Parameter			params[];


  public Clock() {
    super(2);

    state=0;
    nextchg=waits[state];
    setPinValue(states[state], 0);
    setPinValue(istates[state], 1);

    size = new Dimension(3, 3);
  }


  public String getComponentName() {
    return("Clock");
  }

  public String getDescription() {
    return("Clock element with variable frequency.");
  }

  public String getPinDescription(int num) {
    throw new InternalError();
  }

  public int getParameterCount() { return(4); }

  public fips.sim.dig.base.Parameter[] getParameters() {
    if(params==null) {
      params=new Parameter[4];
      for(int i=0; i<4; i++) {
        params[i]=new TimeParameter(paramnames[i], "",
                     java.math.BigInteger.valueOf(defwaits[i]),
                     java.math.BigInteger.valueOf(waits[i]),
                     java.math.BigInteger.ZERO,
                     java.math.BigInteger.valueOf(Long.MAX_VALUE));
      }
    }

    return(params);
  }


  public void setParameter(int num, Object val) {
    //TRACE (".setParameter(num="+num+", val="+val+").");

    if(!(val instanceof java.math.BigInteger)) {
      throw new IllegalArgumentException();
    }

    java.math.BigInteger bval=(java.math.BigInteger)val;
    if((bval.signum()<0)||(bval.bitLength()>63)) {
      throw new IllegalArgumentException();
    }

    waits[num]=bval.longValue();
  }







  public long getNextStep() {
    return(nextchg);
  }


/** Do the following calculation step. If cycles is too big,
 *  IllegalArgumentException is thrown
 */
  public void doStep(long cycles) {
    if(cycles>nextchg) throw new IllegalArgumentException();

    nextchg-=cycles;
    if(nextchg==0) {
      while(nextchg==0) {
        state++;
        if(state==4) state=0;
        nextchg=waits[state];
      }

      setPinValue(states[state], 0);
      setPinValue(istates[state], 1);

      needsupdate=true;
    }
  }




/** Reset cyclecount, params, etc...
 */
  public void reset() {
    throw new InternalError();
  }



  public void writeState(java.io.ObjectOutputStream oos)
                         throws java.io.IOException {
    throw new InternalError();
  }
 
 
  public void readState(java.io.ObjectInputStream ois)
                        throws java.io.IOException {
    throw new InternalError();
  }





/** Paint the clock component. The component has two pin
 *  connections at the bottom,
 */
  public void repaint(Graphics g) {
    //NEVER (".repaint().");

    Dimension size=getSize();
    g.setColor(java.awt.Color.black);
    g.fillRect(offset.x, offset.y, size.width<<scale, size.height<<scale);

    if(scale>2) {
      g.setColor(java.awt.Color.white);
      g.drawRect(offset.x+2, offset.y+2,
                        (size.width<<scale)-5, (size.height<<scale)-5);
    }

    update(g);
  }


  public void update(Graphics g) {
    if(scale>2) {
      g.setColor(colors[states[state]]);
      g.fillRect(offset.x+(1<<(scale-2)), offset.y+(1<<scale),
                        1<<scale, 1<<scale);

      g.setColor(colors[istates[state]]);
      g.fillRect(offset.x+(7<<(scale-2)), offset.y+(1<<scale),
                        1<<scale, 1<<scale);
    }
    needsupdate=false;
  }





  public void doClick(int x, int y, int button) {
  }

  public java.awt.Point getPinPos(int num) {
    if((num<0)||(num>=2)) throw new IllegalArgumentException();

    java.awt.Point ret=new java.awt.Point();
    if((orientation&1)==0) {
      ret.x=1+(num^(orientation>>1));
      ret.y=(orientation==2?0:3);
    } else {
      throw new InternalError();
    }
    return(ret);
  }


  public Object clone() {
    Clock clone=new Clock();
    clone.size=(Dimension)(size.clone());
    clone.orientation=orientation;
 
    clone.name=name;
    clone.state=state;
    clone.nextchg=nextchg;

    clone.waits=(long[])(waits.clone());
    clone.colors=(java.awt.Color[])(colors.clone());
    return(clone);
  }


}



