package fips.sim.dig.comp;

/** Display part of the LCD display. It contains the display
 *  data and the character generator ram. It gets the shift and
 *  show cursor commands from the LCDDisplay controller component.
 */



public class LCDDisplayGUI {


  public final static int	TYPE_SMALL1x8   = 0;    //
  public final static int	TYPE_BIG1x8     = 1;    // 1 line, 10x5, 8 char
  public final static int	TYPE_SMALL2x8   = 2;    // 2 line, 8x5   8 char
 
  public final static int	TYPE_SMALL1x16  = 3;
  public final static int	TYPE_BIG1x16    = 4;
  public final static int	TYPE_SMALL2x16  = 5;


/** Real type of the display. This value affects the gui
 *  presentation of the component.
 */
  int				realdisplaytype;


/** The registered display type. This is the display type value
 *  written dynamically into the controller and affects the
 *  number of characters displayed and the ddram read/write
 *  behaviour.  
 */
  int				displaytype;


/** Scale factor to allow bigger and better readable display
 */
  int				factor;


  boolean			hilight;

  boolean			contrast;


/** Content of the data ram. The storage structure is very
    tricky. It contains 80 bytes (0x50), the first 40 from
    address 0 (0x0) to 39 (0x27), the next 40 from address
    64 (0x40) to 103 (0x67). To allow easier access the array
    contains 128 bytes, writing to unimplemented bytes will
    result in an error message.
*/
  byte				ddram[];


/** Current cursor pos in ddram
*/
  int				cursor;



/** Shift factor for the display. Can be 0 to 0x4F for 1 line
 *  display and 0 to 0x27 for 2 line display.
 */
  int				shift;






  java.awt.Graphics		graphics;
  java.awt.Color		background=new java.awt.Color(0xC0C0C0);
  java.awt.Color		bordercolor=new java.awt.Color(0x0);
  java.awt.Color		dispoff=new java.awt.Color(0x808080);
  java.awt.Color		dispon=new java.awt.Color(0xFFFF00);
  java.awt.Color		pixcolor=new java.awt.Color(0x0);

  int				dotsize;
  
/**Size of the component in units. 
*/
  java.awt.Dimension		size;


/** Size of the component in pixels.
 */
  java.awt.Dimension		realsize;

/** Size of the inner part of the display with boarders measured
 *  in dots.
 */
  java.awt.Dimension		displaysize;

/** Start point of border around display area
*/
  java.awt.Point		border;

/** Width, height of border
*/
  java.awt.Dimension		bordersize;

/** Coordinate of first pixel of first char in first row
*/
  java.awt.Point		dispstart;








  long				cg5x8data[]={
		0x5555555555L
                   };






/** 
 */
  public LCDDisplayGUI() {
    int i;

    displaytype=realdisplaytype=0;

    ddram=new byte[80];
    cursor=0;
    shift=0;

    hilight=false;
    contrast=false;
  }





/** Set the type parameters of this display. This also does
 *  the calculation of border and component size.
 *  @param scale default scale for best resolution of the display
 *  @param factor enlargement factor for the inner display part
 *  @throws IllegalArgumentException if the component currently
 *  has a graphics context associated.
 */
  public void setType(int scale, int factor, int realdisplaytype) {
    if(graphics!=null) {
      throw new IllegalArgumentException("Can't alter type while displayed");
    }

    int			w, h;

    this.realdisplaytype=realdisplaytype;
    this.factor=factor;

    displaysize=getDisplaySize(realdisplaytype);


// 2*4 pix left,right, 2*2 lcd boarder
    w=(displaysize.width<<factor)+12;
 
// 2*14 pix upper, lower, 2*2 lcd boarder
    h=(displaysize.height<<factor)+32;
 
    size=new java.awt.Dimension((w+15)>>4, (h+15)>>4);
    if(size.width<8) size.width=8;

    setScale(scale);
  }




/** Get the size of this component in units.
 */
  public java.awt.Dimension getSize() {
    return(size);
  }




  public void setContrast(boolean val) {
    if(contrast==val) return;
    contrast=val;
    if(graphics!=null) fullUpdateGraphics();
  }

  public boolean getContrast() {
    return(contrast);
  }



  public void setHilight(boolean val) {
    if(hilight==val) return;
    hilight=val;
    if(graphics!=null) fullUpdateGraphics();
  }
 
  public boolean getHilight() {
    return(hilight);
  }





/** Calculate border sizes for given settings
 */
  void setScale(int scale) {
    int			i, sscale;


    sscale=scale+factor;

// this is the space the component has
    realsize=new java.awt.Dimension(size.width<<scale, size.height<<scale);


// calculate dotsize for 2*4 pix boarder, 2*2 pix frame
    i=(realsize.width-12)/displaysize.width;
    dotsize=(realsize.height-32)/displaysize.height;
    if(i<dotsize) dotsize=i;


    if(dotsize==0) {
// no display possible, check if no boarder is allowed
// disable displaying of boarder
      border=null;
      bordersize=null;

      i=realsize.width/displaysize.width;
      dotsize=realsize.height/displaysize.height;
      if(i<dotsize) dotsize=i;

      if(dotsize!=0) {
        if(((dotsize*displaysize.width+6)<realsize.width)&&
           ((dotsize*displaysize.height+6<realsize.height))) {
 // boarder possible

          bordersize=new java.awt.Dimension(4+dotsize*displaysize.width,
                                            4+dotsize*displaysize.height);
          border=new java.awt.Point((realsize.width-bordersize.width)>>1,
                                    (realsize.height-bordersize.height)>>1);
        }
      } else {
        //WARNING ("#: display too small, no gui possible");
      }

    } else {
      bordersize=new java.awt.Dimension(4+dotsize*displaysize.width,
                                        4+dotsize*displaysize.height);

      border=new java.awt.Point((realsize.width-bordersize.width)>>1,
                                (realsize.height-bordersize.height)>>1);
    }


    if(dotsize!=0) {
// cause display width in dsize includes boarders,
// subtract 4 * dotsize from dsize
      dispstart=new java.awt.Point();
      dispstart.x=(realsize.width-dotsize*displaysize.width+4*dotsize)>>1;
      dispstart.y=(realsize.height-dotsize*displaysize.height+4*dotsize)>>1;
    }
  }






// ************************************************************************* //
//
//			Internal Methods				     //
//
// ************************************************************************* //




/** Get a value out of the 80 byte ddram. If the display has only
 *  1 line, addr accesses all 0x50 bytes in one linear block.
 *  When an 2-line display is used, the first 40 bytes are addressed
 *  from 0 to 0x27, the next 40 from 0x40 to 0x67. If the address
 *  doesn't point to an valid address in memory -1 is returned.
 */
  int getRamData(int addr) {
    if(getRows(displaytype)==1) {
      if(addr>0x4F) return(-1);
      return(ddram[addr]);
    }

    if((addr>0x67)||((addr&0x3F)>0x27)) return(-1);
    if(addr>0x27) addr-=0x18;

    return(ddram[addr]);
  }



/** Get the number of rows for this type of display
 */
  int getRows(int type) {
    switch(type) {
      case TYPE_SMALL1x8:
      case TYPE_BIG1x8:
      case TYPE_SMALL1x16:
      case TYPE_BIG1x16:
        return(1);
      case TYPE_SMALL2x8:
      case TYPE_SMALL2x16:
        return(2);
      default:
        throw new IllegalArgumentException();
    }
  }






/** Get the number characters in one row
 */
  int getCharsPerRow(int type) {
    switch(type) {
      case TYPE_SMALL1x8:
      case TYPE_BIG1x8:
      case TYPE_SMALL2x8:
        return(8);
      case TYPE_BIG1x16:
      case TYPE_SMALL1x16:
      case TYPE_SMALL2x16:
        return(16);
      default:
        throw new IllegalArgumentException();
    }
  }




/** Get the height of the used font
 */
  int getCharHeight(int type) {
    switch(type) {
      case TYPE_SMALL1x8:
      case TYPE_SMALL2x8:
      case TYPE_SMALL1x16:
      case TYPE_SMALL2x16:
        return(8);
      case TYPE_BIG1x8:
      case TYPE_BIG1x16:
        return(10);
      default:
        throw new IllegalArgumentException();
    }
  }





/**Get the number of dots in the display of given type inclusively
   the boarder dots.
*/
  java.awt.Dimension getDisplaySize(int type) {
    switch(type) {
      case TYPE_SMALL1x8:
// w = 8*5 + 7*1 + 2*2          h= 1*8 + 2*2
        return(new java.awt.Dimension(51,12));
      case TYPE_BIG1x8:
      case TYPE_SMALL1x16:
      case TYPE_BIG1x16:
      case TYPE_SMALL2x8:
        throw new IllegalArgumentException();
      case TYPE_SMALL2x16:
// w = 16*5 + 15*1 + 2*2        h= 2*8 + 1*1 + 2*2
        return(new java.awt.Dimension(99, 21));
      default:
        throw new IllegalArgumentException();
    }
  }





// ************************************************************************* //
//
//			GUI Methods					     //
//
// ************************************************************************* //


/** Init the graphics with given scale and calculate the values
 *  of boarders and spaces.
 *  @param scale the real scale of the display
 */
  public void setGraphics(java.awt.Graphics g, int scale) {

    setScale(scale);
    graphics=g;
 
    if(g==null) return;

    initGraphics();
  }



/** Do the full update of the gui.
 */
  void initGraphics() {
    //TRACE ("#: graphics init triggered");

    graphics.setColor(background);

    graphics.fillRect(0, 0, realsize.width, realsize.height);

    if(border!=null) {
      graphics.setColor(bordercolor);
      graphics.drawRect(border.x, border.y, bordersize.width-1,
                        bordersize.height-1);
      graphics.drawRect(border.x+1, border.y+1, bordersize.width-3,
                        bordersize.height-3);
    }

    if(dotsize>0) fullUpdateGraphics();
  }





/** Draw the content of the display region completely new.
 *  This method should be called after major parameter changes
 *  like switching contrast on/off or changing the font type
 */
  void fullUpdateGraphics() {
    java.awt.Color		bg;
    java.awt.Dimension		dsize;
    int				x, y;

    bg=(hilight?dispon:dispoff);

    graphics.setColor(bg);
    graphics.fillRect(dispstart.x-(dotsize<<1), dispstart.y-(dotsize<<1),
                      displaysize.width*dotsize, displaysize.height*dotsize);

    if(!contrast) return;

// now display is clear, paint according to displaytype

    for(y=0; y<getRows(displaytype); y++) {
      for(x=0; x<getCharsPerRow(displaytype); x++) {
        dispCharAt(x, y);
      }
    }
  }








/** Display a character from the ddram.
 */
  void dispCharAt(int px, int py) {
    int			sx, sy, x, y, rows;
    int			i, j;
    int			addr;
    long		cdata;
    java.awt.Color	bg;
    boolean		pix;

    //TRACE (".dispCharAt("+px+", "+py+").");

    if(px>=getCharsPerRow(realdisplaytype)) return;
    if(py>=(rows=getRows(realdisplaytype))) return;

// calculate address from pos+shift
    addr=(py!=0 ? 40 : 0);
    x=px+shift;
    if(rows==1) x=x%0x50;
    else x=x%0x28;
    addr+=x; 

// start of character
    sx=dispstart.x+px*dotsize*6;
    sy=dispstart.y+py*dotsize*(getCharHeight(displaytype)+1);

    bg=(hilight?dispon:dispoff);

    if(getCharHeight(displaytype)==8) {
      if(!contrast) {
        graphics.fillRect(sx, sy, dotsize*5, dotsize*8);
        return;
      }

      i=ddram[addr]&0xFF;
      if(i>=cg5x8data.length) {
        //WARNING ("#: no 5x8 cgdata for "+i+" available");
        i=0;
      }

      cdata=cg5x8data[i];
      for(i=0; i<8; i++) {
        y=sy+i*dotsize;
        for(j=0; j<5; j++) {
          x=sx+j*dotsize;

          pix=((cdata&(1L<<(j+i*5)))!=0);

          if(dotsize<3) {	// one or two pix dots
            graphics.setColor(pix?pixcolor:bg);
            graphics.fillRect(x, y, dotsize, dotsize);
          } else if(dotsize<5) {
            graphics.setColor(bg);
            if(pix) {
              graphics.drawLine(x, y, x+dotsize-1, y);
              graphics.drawLine(x, y, x, y+dotsize-1);
              graphics.setColor(pixcolor);
              graphics.fillRect(x+1, y+1, dotsize-1, dotsize-1);
            } else graphics.fillRect(x, y, dotsize, dotsize);
          } else { // use 1 pix of dot for border
            graphics.setColor(bg);
            if(pix) {
              graphics.drawRect(x, y, dotsize-1, dotsize-1);
              graphics.setColor(pixcolor);
              graphics.fillRect(x+1, y+1, dotsize-2, dotsize-2);
            } else graphics.fillRect(x, y, dotsize, dotsize);
          }
        }
      }
      return;
    }

    throw new InternalError("5x10 font not supported.");

  }











}






