package fips.sim.dig.comp.fsm;


/** Store the conditions to enter a new state. The condition
 *  contains value and timer conditions.
 */

public class ChangeCondition {


/** Check if a state variable is in a set of given variables.
 */
  public final static int	COND_ISIN		= 0;
  public final static int	COND_ISNOTIN		= 1;

  public final static int	COND_ISGREATER		= 2;
  public final static int	COND_ISLESS		= 3;

/** Check if one state variable equals another
 */
  public final static int	COND_EQUALSVAR		= 4;
  public final static int	COND_NOTEQUALSVAR	= 5;

  public final static int	COND_VARISGREATER	= 6;
  public final static int	COND_VARISLESS		= 7;



  public final static int	TIMER_ISGREATEREQUAL	= 0;
  public final static int	TIMER_ISLESSEQUAL	= 1;



  FSMachine			fsmachine;

/** List of conditions on state variables. Each condition is
 *  stored in one array of int, the first field containing the
 *  type of condition, the next the state variable to test and
 *  the following additional parameters.
 */
  public int			valchecks[][];

/** List of integers, two for each timer condition. The first
 *  contains the type of timer condition, the second the timer
 *  affected. There is one value in the timerlimits corresponding
 *  to each value pair in the timer list.
 */
  int				timers[];
  long				timerlimits[];

/** Method to check if this condition is true. The checkmethod
 *  is intended to calculate complicated conditions and has to
 *  return a long value containing the steps until the condition
 *  becomes true. The -1 value denotes that currently a state
 *  transition is not possible due to state variables, 0 that
 *  the condition is allready true.
 */
  java.lang.reflect.Method	checkmethod;


  String			newstatename;
  StateDef			newstate;



  public ChangeCondition(FSMachine fs, String newstatename) {
    fsmachine=fs;
    this.newstatename=newstatename;
  }



/** Get the next state to enter if this condition gets true.
 */
  public StateDef getNextState() {
    //TRACE ("#: next state is "+newstatename);
    if(newstate==null) {
//      newstate=fsmachine.getStateForName(newstatename);
      throw new InternalError();
    }
    return(newstate);
  }




/** Get the time until a timer driven condition will get true
 *  for the current set of state variables
 *  @returns -1 if condition can't get true for current state variables
 *  0 if condition is allready true and a value > 0 if the
 *  condition will get true in that amount of cycles
 */
  public long getTime() {
    long	steps;
    int		i, j, val;

    steps=0;

    if(valchecks!=null) {
      for(i=0; i<valchecks.length; i++) {
        switch(valchecks[i][0]) {
          case COND_ISIN:
            val=fsmachine.statevars[valchecks[i][1]];
            for(j=2; j<valchecks[i].length; j++) {
              if(val==valchecks[i][j]) break;
            }
            if(j==valchecks[i].length) {
// condition can't get true
              return(-1);
            }
            break;
          default:
            throw new InternalError("Invalid check type "+valchecks[i][0]);
        }
      }
    }

    if(timers!=null) {
      throw new InternalError();
    }

    if(checkmethod!=null) {
      throw new InternalError();
    }

    return(steps);
  }


}



