package fips.sim.dig.comp.fsm;



/** Holder describing one action to be taken. The action may
 *  set or copy state variables, reset, increase or decrease
 *  a timer. The increasing or decreasing of timers is only
 *  usefull in a doStep method call because this changes the
 *  value of the counter for the cycle count of the doStep
 *  method. Enter and leave methods don't get a cycles argument,
 *  they are executed indefinitly fast in 0 cycles, so timers
 *  won't change.
 */
public class StateAction {
  public final static int	ACTION_SETVAR	= 0;
  public final static int	ACTION_COPYVAR	= 1;
  public final static int	ACTION_SETTIMER	= 5;


/** Increase a timer for the value of steps in the doSteps method.
 *  So this action has no effect in an enter or leave action
 *  environment. The timerval is a maximum or minimum value
 *  for increasing, decreasing.
 */
  public final static int	ACTION_INCTIMER	= 6;
  public final static int	ACTION_DECTIMER	= 7;

  int		action;

/** Stores destination state variable or timer, depending on
 *  current action type.
 */
  int		varnum;
  int		newval;
  int		copyvar;

  long		timerval;



  public StateAction(int type, int varnum, int newval, int copyvar,
                     long timerval) {
    this.action=type;
    this.varnum=varnum;
    this.newval=newval;
    this.copyvar=copyvar;

    this.timerval=timerval;
  }



/** Perform actions on the timers and state variables in the
 *  final state machine. The results are written into the newvars
 *  and newtimers arrays.
 */
  public void performAction(FSMachine fs, long cycles) {
    long	l;
    switch(action) {
      case ACTION_SETTIMER:
        fs.newtimers[varnum]=timerval;
        break;
      case ACTION_DECTIMER:
        l=fs.timers[varnum]-cycles;
        if(l>fs.timers[varnum]) l=Long.MIN_VALUE;
        if(l<timerval) l=timerval;
        fs.newtimers[varnum]=l;
        break;
      default:
        throw new InternalError("Illegal action "+action);
    }
  }

}





