package fips.sim.dig.edit;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.event.MouseEvent;
import java.math.BigInteger;
import java.util.ArrayList;
import java.util.logging.Logger;

import javax.swing.JToolTip;

import fips.sim.dig.base.Component;
import fips.sim.dig.comp.Pin;
import fips.sim.dig.edit.configure.ComponentConfigurator;

/** All methods that modify the circuit structure have
 *  to be synchronized.
 */
public class CircuitCanvas extends javax.swing.JComponent
// public class CircuitCanvas extends javax.swing.JPanel
                           implements java.awt.event.ComponentListener,
                                      java.awt.event.MouseListener,
                                      java.awt.event.MouseMotionListener,
                                      GateSelectionListener,
                                      fips.sim.dig.base.TimerListener {

  private Logger			logger;

/** Current scale
 */
  int					scale;


/** Start of drawing area in pixels, not gridunits. If the
 *  circuit is displayed with a start value, then the pixel
 *  at 0,0 when shifted via start, would lie at start if
 *  shifted with 0,0.
 *  If you have a relative position in the window, add start
 *  to get the global position
 */
  Point					start;



/** Size of the drawing area in units
 */
  java.awt.Dimension			size;




  fips.sim.dig.base.Timer		timer;


/** Array holding list of currently displayed components
 */
  Component				components[];


/** Positioning data of each visible component containing
 *  x, y, width, height in the grid coord system.
 *  The x-position coords are sorted.
 */
  int					positions[];

  int					componentcnt;

/** ArrayList holding the pins for each line potential.
 *  As soon as a new potential is created, a new pin
 *  object has to be added to this list. The number of
 *  elements in this list reflects the number of different
 *  potentials in the circuit.
 */
  ArrayList				pins;


  int					linecnt;

/** Stores lines with x1,y1 end x2,y2 and pot nr The x1
 *  values are sorted, x2>=x1. The coords are grid-pos
 *  numbers, not pixel numbers
 */
  int					lines[];


  int					connectcnt;
/** Points where lines are connected with other lines.
 *  Contains x, y, pot numbers
 */
  int					connectpoints[];




/** Starting point for drawing a line.
 */
  Point					linestart;


  private final static int		CURSOR_NONE	=  0;
  private final static int		CURSOR_GATE	=  1;

  int					cursortype=0;


/** Gate image for gatecursor
 */
  java.awt.Image			gatecursor;

/** Content of circuit canvas region before adding cursor image.
 */
  java.awt.Image			oldregion;

  Component				currgate;

/** Absolute gate position in units
 */
  Point					gateposition;



  ComponentConfigurator			cconfig;



  public CircuitCanvas() {
    super();

    logger=Logger.getLogger("digsim.edit");

//    setUI(CircuitCanvasUI.createUI(this));
    setUI(null);

    scale=4;

    addMouseListener(this);
    addMouseMotionListener(this);
    addComponentListener(this);

    setOpaque(true);  // component is solid filled, nothing shows through
//    setDoubleBuffered(false);


    javax.swing.ToolTipManager.sharedInstance().registerComponent(this);

    start = new Point(0, 0);

// components
    components=new Component[16];
    positions=new int[components.length<<2];
    componentcnt=0;

// lines
    linecnt=0;
    lines=new int[5<<4];

// pins
    pins=new ArrayList();

// connectors
    connectcnt=0;
    connectpoints=new int[3*16];

    timer = new fips.sim.dig.base.Timer();

    timer.addTimerListener(this);

  }






// ************************************************************************* //
//									     //
//			Interaction with simulator			     //
//									     //
// ************************************************************************* //






// ************************************************************************* //
//									     //
//			Internal Methods				     //
//									     //
// ************************************************************************* //


/** Get the grid position nearest to pixel pos coord.
 */
  private Point getNearestGridPoint(int x, int y) {

    Point pos=new Point(x+start.x, y+start.y);

    if((pos.x&(-1<<scale))!=((pos.x+(1<<(scale-1)))&(-1<<scale))) {
      pos.x=(pos.x>>scale)+1;
    } else pos.x=pos.x>>scale;

    if((pos.y&(-1<<scale))!=((pos.y+(1<<(scale-1)))&(-1<<scale))) {
      pos.y=(pos.y>>scale)+1;
    } else pos.y=pos.y>>scale;

    return(pos);
  }




 

//**************************************************************************//
//
//			GateSelectionListener impl			    //
//
//**************************************************************************//


  public void gateSelected(Component comp) {
    //TRACE (".gateSelected(comp="+comp.getName()+").");

    cursortype=CURSOR_GATE;
    gatecursor=ComponentTools.getImage(comp, scale, comp.getOrientation());
    oldregion=null;
    currgate=comp;
    gateposition=null;

  }




//**************************************************************************//
//
//			ComponentListener impl				    //
//
//**************************************************************************//


 public void componentResized(java.awt.event.ComponentEvent ce) {
   logger.finer(".componentResized(ce).");
   size=getSize();
 } 

 public void componentMoved(java.awt.event.ComponentEvent ce) {
 }

 public void componentShown(java.awt.event.ComponentEvent ce) {
 }

 public void componentHidden(java.awt.event.ComponentEvent ce) {
 }





// ************************************************************************* //
//
//			Mouse Listener Implementation			     //
//
// ************************************************************************* //



  public synchronized void mouseClicked(MouseEvent me) {
    logger.fine(".mouseClicked() at "+me.getPoint().x+"x"+me.getPoint().y+
                ", buttons "+me.getButton());

    switch(cursortype) {
      case CURSOR_NONE:
        int cpos=getComponentIndexAtGridPos((me.getX()+start.x)>>scale,
                                            (me.getY()+start.y)>>scale);

        if(cpos<0) {
          logger.finest("No component at "+me.getX()+","+me.getY());
          return;
        }


        if(me.getButton()==me.BUTTON1) {
          logger.finer("Sending event to component "+
                       components[cpos].getName());

          components[cpos].doClick(
                             me.getX()+start.x-(positions[cpos<<2]<<scale),
                             me.getY()+start.y-(positions[(cpos<<2)+1]<<scale),
                             me.getModifiers());
          break;
        }

        if(me.getButton()==me.BUTTON3) {
          logger.fine("Triggered configure of component "+components[cpos].getName());
          BigInteger timerend=null;
          if(timer.isRunning()) {
            timerend=timer.getTimerEnd();
            timer.stopTimer();
          }

// Disconnect inputs to allow size changes of the component
          disconnectComponent(cpos);

          if(cconfig==null) cconfig=new ComponentConfigurator();

// let the configurator do its work
          if(cconfig.configure(components[cpos])) {

// update the size cache in components (size maybe changed)
            positions[(cpos<<2)+2]=components[cpos].getSize().width;
            positions[(cpos<<2)+3]=components[cpos].getSize().height;
//            components[cpos].repaint();
            logger.severe("No repaint check yet");
          }

// Reconnect the component
          connectComponent(cpos);

          if(timerend!=null) {
            throw new InternalError();
          }
        }

        break;
      case CURSOR_GATE:
        if(me.getButton()==me.BUTTON1) {
          Point pos;
          pos=getNearestGridPoint(me.getX()-
                                      (currgate.getSize().width<<(scale-1)),
                              me.getY()-(currgate.getSize().height<<(scale-1)));
// add the gate at current pos
          Component c=(Component)(currgate.clone());
          addComponent(c, pos.x, pos.y);
          repaintGrid(pos.x, pos.y, c.getSize().width, c.getSize().height);
          break;
        }

        if(me.getButton()==me.BUTTON3) {
// cancel adding of gates
          cursortype=CURSOR_NONE;
          showGateCursor(null);
          break;
        }

        break;
      default:
        throw new InternalError("Unknown cursor type "+cursortype);
    }
  }



  public void mousePressed(MouseEvent me) {
    logger.fine(".mousePressed().");

    switch(cursortype) {
      case CURSOR_NONE:

        if(me.getButton()==me.BUTTON1) {
          linestart=getNearestGridPoint(me.getX(), me.getY());
        } else linestart=null;

        break;
      case CURSOR_GATE:
        break;
      default:
        throw new InternalError("Unknown cursor type "+cursortype);
    }
  }


  public void mouseReleased(MouseEvent me) {
    switch(cursortype) {
      case CURSOR_NONE:
        if(linestart==null) break;

        Point pos=getNearestGridPoint(me.getX(), me.getY());
        while(!pos.equals(linestart)) {
// draw the line
          int linenum=-1;

          Point midpos=getLineMidPosition(linestart, pos);

          linenum=addLine(linestart.x, linestart.y, midpos.x, midpos.y);

          linestart=midpos;
        }

        repaintCircuitCanvas();

        break;
      case CURSOR_GATE:
        break;
      default:
        throw new InternalError("Unknown cursor type "+cursortype);
    }
  }


  public void mouseEntered(MouseEvent me) {

    switch(cursortype) {
      case CURSOR_NONE:
        break;
      case CURSOR_GATE:
        showGateCursor(me.getPoint());
        break;
      default:
        throw new InternalError("Unknown cursor type "+cursortype);
    }
  }



  public void mouseExited(MouseEvent me) {

    switch(cursortype) {
      case CURSOR_NONE:
        break;
      case CURSOR_GATE:
        showGateCursor(null);
        break;
      default:
        throw new InternalError("Unknown cursor type "+cursortype);
    }

  }




// ************************************************************************* //
//
//			Mouse Listener Implementation			     //
//
// ************************************************************************* //


  public void mouseDragged(MouseEvent me) {
  }

  public void mouseMoved(MouseEvent me) {
    switch(cursortype) {
      case CURSOR_NONE:
        break;
      case CURSOR_GATE:
        showGateCursor(me.getPoint());
        break;
      default:
        throw new InternalError("Unknown cursor type "+cursortype);
    }
  }






//**************************************************************************//
//
//			General Purpose methods
//		enlarging/clearing of arrays, etc ...			    //
//
//**************************************************************************//


/** Delete all elements of this circuit.
 */
  private synchronized void deleteCircuit() {
    for(int i=0; i<componentcnt; i++) components[i]=null;
    componentcnt=0;

    pins.clear();
    linecnt=0;
    connectcnt=0;
  }



/** Set the lenght of the linearray to hold newlength line-coord/pot
 *  points.
 */
  private synchronized void setLineArrayLength(int newlen) {
    logger.fine(".setLineArrayLength(newlen="+newlen+").");

    int ibuf[]=new int[newlen*5];
    System.arraycopy(lines, 0, ibuf, 0, linecnt*5);
    lines=ibuf;
  }


  private synchronized void setConnectPointsArrayLength(int newlen) {
    int ibuf[]=new int[newlen*3];
    System.arraycopy(connectpoints, 0, ibuf, 0, connectcnt*3);
    connectpoints=ibuf;
  }


  private synchronized void setComponentsArrayLength(int newlen) {
    Component cbuf[]=new Component[newlen];
    System.arraycopy(components, 0, cbuf, 0, componentcnt);
    components=cbuf;

    int ibuf[]=new int[newlen<<2];
    System.arraycopy(positions, 0, ibuf, 0, componentcnt<<2);
    positions=ibuf;
  }



// ************************************************************************* //
//
//				Line management				     //
//				synchronized				     //
//									     //
// ************************************************************************* //




/** Get the mid position of a line. If the line from start to
 *  end is horizontal, vertical or 45deg end is returned.
 *  In any other case a mid point is returned, so that the
 *  midpoint's x or y coordinate will match one of end's x,y
 */
  private Point getLineMidPosition(Point start, Point end) {
    int dx, dy;
    int ddiff;

    ddiff=Math.abs(dx=end.x-start.x)-Math.abs(dy=end.y-start.y);
    if((ddiff==0)||(dx==0)||(dy==0)) return(end);

    if(ddiff>0) { // dx part is longer than dy part
      return(new Point(end.x-ddiff*signum(dx), end.y)); 
    } else {
      return(new Point(end.x, end.y+ddiff*signum(dy))); 
    }
  }



/** Add a line to the array of linedata. This method doesn't
 *  affect any other connection data.
 *  @returns the pos of the new line in the array.
 */
  private int addLineToLineArray(int fx, int fy, int tx, int ty, int pot) {
    //TRACE (".addLineToLineArray(fx="+fx+", fy="+fy+", tx="+tx+",ty="+ty+
    //TRACE  ", pot="+pot+").");

    if(tx<fx) throw new IllegalArgumentException();
    if((tx==fx)&&(ty<=fy)) throw new IllegalArgumentException();

    int pos;
    for(pos=0; pos<linecnt; pos++) {
      if(lines[pos*5]>fx) break;
    }

    int offs=pos*5;

    if(linecnt*5==lines.length) { // enlarge line array
      setLineArrayLength(linecnt+(linecnt>>2)+16);

      //ERROR ("#: more code needed????");
    }


    if(pos!=linecnt) {
// move values at offs 5 vals back
      System.arraycopy(lines, offs, lines, offs+5, linecnt*5-offs);
    }

    linecnt++;

    lines[offs]=fx; lines[offs+1]=fy; lines[offs+2]=tx; lines[offs+3]=ty;
    lines[offs+4]=pot;

    return(pos);
  }





/** Remove a line from the array of linedata. This method doesn't
 *  affect any other connection data.
 */
  private void removeLineFromLineArray(int pos) {
    //TRACE (".removeLineFromLineArray(pos="+pos+").");

    linecnt--;

    if(pos!=linecnt) {
// move values 5 pos to front
      System.arraycopy(lines, (pos+1)*5, lines, pos*5, (linecnt-pos)*5);
    }

  }








/** Add a line and manage the pins.
 *  @returns the number of the new line containing the piece
 *  from fx,fy to tx,ty. The linenumber may refer to a longer
 *  line if the new line was merged with other lines
 */
  private synchronized int addLine(int fx, int fy, int tx, int ty) {
    //TRACE (".addLine(fx="+fx+", fy="+fy+", tx="+tx+", ty="+ty+").");

    if(fx>tx) {
      int i=fx; fx=tx; tx=i;
      i=fy; fy=ty; ty=i;
    } else if((fx==tx)&&(fy>ty)) {
      int i=ty; ty=fy; fy=i;
    }


      int frompot=getPotentialAtGridPos(fx, fy);
      int topot=getPotentialAtGridPos(tx, ty);

      if(frompot<0) frompot=topot;

// frompot contains pot of both ends of line,
// add line with this pot. If pot < 0 create new pin
      if(frompot<0) {
        frompot=pins.size();
        pins.add(new Pin());
      }

      Pin p=(Pin)pins.get(frompot);

// add line
      int pos1=addLineToLineArray(fx, fy, tx, ty, frompot);

      connectLinesAt(fx, fy);
      int pot=connectLinesAt(tx, ty);


// remove additional lines if these lines end on the line
// from fx to fy.

      //ARM ("#: checking for lines with same dir under line "+pos1);

      int dirx=signum(tx-fx);
      int diry=signum(ty-fy);

      for(int pos2=0; pos2<linecnt; pos2++) {
        if(pos2==pos1) continue;

        int posoffs1=pos1*5;
        int posoffs2=pos2*5;

        int dx=signum(lines[posoffs2+2]-lines[posoffs2]);
        int dy=signum(lines[posoffs2+3]-lines[posoffs2+1]);

        if(((dirx==dx)&&(diry==dy))||((dirx==-dx)&&(diry==-dy))) {
          logger.finest("Line "+pos2+" has same dir");

// dir ok, now check if one of points is on line

          boolean p1=isPointOnLine(fx, fy, pos2);
          boolean p2=isPointOnLine(tx, ty, pos2);
          boolean p3=isPointOnLine(lines[posoffs2],   lines[posoffs2+1], pos1);
          boolean p4=isPointOnLine(lines[posoffs2+2], lines[posoffs2+3], pos1);

          if(p1||p2||p3||p4) {

            //ARM ("#: merging line "+lines[posoffs1]+"x"+lines[posoffs1+1]+'('+
            //ARM  p1+") - "+lines[posoffs1+2]+"x"+lines[posoffs1+3]+'('+
            //ARM  p2+") and line "+lines[posoffs2]+"x"+lines[posoffs2+1]+'('+
            //ARM  p3+") - "+lines[posoffs2+2]+"x"+lines[posoffs2+3]+'('+
            //ARM  p4+").");

// merge the two pins and pots
            if(lines[posoffs1+4]!=lines[posoffs2+4]) {
              //ARM ("#: merging pots "+lines[posoffs1+4]+" with "+
              //ARM  lines[posoffs2+4]);

              throw new InternalError();
            }

// set fx,fy tx,ty to end points of lines
// rempoints is list of points to remove, 

            Point removepts[]=new Point[2];
            int rcnt=0;

            if(p1&&p2&&p3&p4) { // identical lines, no removepts needed
              //ARM ("#: identical lines");
              fx=lines[posoffs1]; fy=lines[posoffs1+1];
              tx=lines[posoffs1+2]; ty=lines[posoffs1+3];
            } else if(!p1) { // p1 is start of line
              fx=lines[posoffs1]; fy=lines[posoffs1+1];
              //ARM ("#: start pnt is "+fx+"x"+fy);
              if(!p2) { // line p3,p4 is under p1,p2
                //ARM ("#: line "+pos1+" is over "+pos2);

                tx=lines[posoffs1+2]; ty=lines[posoffs1+3];
                removepts[rcnt++]=new Point(lines[posoffs2], lines[posoffs2+1]);
                removepts[rcnt++]=new Point(lines[posoffs2+2],
                                                     lines[posoffs2+3]);
              } else if(!p4) {
// line p1,p2 is partially over p3,p4 but p3,p4 is longer
// remove p2,p3 if not the same

                tx=lines[posoffs2+2]; ty=lines[posoffs2+3];
                removepts[rcnt++]=new Point(lines[posoffs1+2],
                                                     lines[posoffs1+3]);
                if((lines[posoffs1+2]!=lines[posoffs2])||
                   (lines[posoffs1+3]!=lines[posoffs2+1])) {
                  removepts[rcnt++]=new Point(lines[posoffs2],
                                                       lines[posoffs2+1]);
                }
              } else { // both lines end on same point
                throw new InternalError();
              }
            } else if(!p3) { // p3 is start of line
              fx=lines[posoffs2]; fy=lines[posoffs2+1];

              //ARM ("#: start pnt is "+fx+"x"+fy);
              if(!p4) { // line p1,p2 is under p3,p4
                //ARM ("#: line "+pos2+" is over "+pos1);

                tx=lines[posoffs2+2]; ty=lines[posoffs2+3];
                removepts[rcnt++]=new Point(lines[posoffs1],
                                                     lines[posoffs1+1]);
                removepts[rcnt++]=new Point(lines[posoffs1+2],
                                                     lines[posoffs1+3]);
              } else if(!p2) {
// line p1,p2 is partially over p3,p4 but p3,p4 is longer
// remove p2,p3 if not the same

                tx=lines[posoffs1+2]; ty=lines[posoffs1+3];
                removepts[rcnt++]=new Point(lines[posoffs2+2],
                                                     lines[posoffs2+3]);
                if((lines[posoffs2+2]!=lines[posoffs1])||
                   (lines[posoffs2+3]!=lines[posoffs1+1])) {
                  removepts[rcnt++]=new Point(lines[posoffs1],
                                                       lines[posoffs1+1]);
                }
              } else { // both lines end on same point
                throw new InternalError();
              }
            }

// set the line with smaller index to new line
            int npos=(pos1<pos2?pos1:pos2);
            lines[npos*5]=fx;   lines[npos*5+1]=fy;
            lines[npos*5+2]=tx; lines[npos*5+3]=ty;

// remove the line with bigger index
            removeLineFromLineArray(pos1<pos2?pos2:pos1);

// deal with the removepoints
            for(int rpoint=0; rpoint<rcnt; rpoint++) {
              //ARM ("#: removing end pty x="+removepts[rpoint].x+
              //ARM  ", y="+removepts[rpoint].y+").");

              pot=connectLinesAt(removepts[rpoint].x, removepts[rpoint].y);
            }

            if(pos2<pos1) pos1=pos2;
          }
        }
      }

      //ARM ("#: check for lines ending on line "+pos1);

// check if any other line ends on the new piece
      for(int i=0; i<linecnt; i++) {
        if(i==pos1) continue;
        int offs=i*5;
        if(lines[offs+4]==pot) continue;
        if(isPointOnLine(lines[offs], lines[offs+1], pos1)) {
          pot=connectLinesAt(lines[offs], lines[offs+1]);
        }

        if(isPointOnLine(lines[offs+2], lines[offs+3], pos1)) {
          pot=connectLinesAt(lines[offs+2], lines[offs+3]);
        }
      }

// add component pins (if any)
      connectComponentsToLine(pos1);

      return(pos1);

  }





/** Connect all lines through x, y. This means merging the pins
 *  of different potentials at the point. The method may not
 *  create a connectpoint if not enough lines go accross x,y.
 *  Different potentials at the connectpoint are merged.
 *  This method may also remove connectpoints if only 1 line
 *  hits x,y or if two lines end here.
 *  @returns the number of the potential at the point or
 *  -1 if none.
 */
  private int connectLinesAt(int x, int y) {
    logger.fine(".connectLinesAt(x="+x+", y="+y+").");

// number of the connection point, if found
    int connectpnt=-1;
    int potnr=-1;

// check if allready connection at x, y
    for(int i=0; i<connectcnt; i++) {
      int offs=i*3;
      if((connectpoints[offs]==x)&&(connectpoints[offs+1]==y)) {
        connectpnt=i;
        potnr=connectpoints[offs+2];
        break;
      }
      if(connectpoints[offs]>x) break;
    }


// no connection at x, y
// search for line(s), if more than one line add connect point

    int conlines=0;		// number of lines at x,y
    boolean endsonly=true;

    for(int i=0; i<linecnt; i++) {
      if(!isPointOnLine(x, y, i)) continue;

      int offs=i*5;

      conlines++;
      logger.finer("Found line "+i+" at x,y");

      if(((lines[offs]!=x)||(lines[offs+1]!=y))&&
         ((lines[offs+2]!=x)||(lines[offs+3]!=y))) {
// line doesn't at x,y
        endsonly=false;
      }

// get pot, create pin, etc ...
      if(potnr==-1) {
        potnr=lines[offs+4];
        continue;
      }

      if(potnr==lines[offs+4]) continue;

      potnr=mergePotentials(lines[offs+4], potnr);
    }

    logger.finer("Found "+conlines+" at x,y ending? "+endsonly);

    if(potnr<0) return(potnr);

    if((conlines==1)||((conlines==2)&&endsonly)) {
// no connectpt needed, remove if if here
      if(connectpnt>=0) removeConnectPoint(x, y);
    } else {
// create connectpnt if not here
      if(connectpnt<0) addConnectPoint(x, y, potnr);
    }

    return(potnr);
  }






/** Check if a given point lies on a line in the linearray
 *  list.
 *  @returns true if the point is in middle of line or at one
 *  end.
 */
  private boolean isPointOnLine(int x, int y, int linenum) {
    logger.finer(".isPointOnLine(x="+x+", y="+y+", linenum="+linenum+").");

    linenum=linenum*5;

    int pdx=x-lines[linenum];
    int pdy=y-lines[linenum+1];

    if((pdx==0)&&(pdy==0)) return(true);

// if line has no valid dir return false
    if((pdx!=0)&&(pdy!=0)&&(Math.abs(pdx)!=Math.abs(pdy))) return(false);

    int ldx=lines[linenum+2]-lines[linenum];
    int ldy=lines[linenum+3]-lines[linenum+1];

// check if lines have the same dir
    if((signum(pdx)!=signum(ldx))||(signum(pdy)!=signum(ldy))) return(false);

// check that length is ok
    return((Math.abs(pdx)<=Math.abs(ldx))&&(Math.abs(pdy)<=Math.abs(ldy)));
  }



  private int signum(int val) {
    return(val==0?0:(val>0?1:-1));
  }





// ************************************************************************* //
//
//			Potential management
//
// ************************************************************************* //




// try to get pot at x,y
  private int getPotentialAtGridPos(int x, int y) {
    for(int i=0; i<linecnt; i++) {
      int offs=i*5;
      if(lines[offs]>x) break;

      if(isPointOnLine(x, y, i)) return(lines[offs+4]);
    }

    return(-1);
  }



/** Merge two potentials.
 *  @returns the number of the merged pots
 */
  public int mergePotentials(int apot, int bpot) {
    //TRACE (".mergePotential(apot="+apot+", bpot="+bpot+").");

    int rempot=apot<bpot?bpot:apot;
    int newpot=apot<bpot?apot:bpot;

// loop over all lines
    for(int lc=0; lc<linecnt; lc++) {
      int offs=lc*5;

      if(lines[offs+4]==rempot) lines[offs+4]=newpot;
    }

// loop over all connection points
    for(int cc=0; cc<connectcnt; cc++) {
      int offs=cc*3;
      if(connectpoints[offs+2]==rempot) connectpoints[offs+2]=newpot;
    }

// get the old and new pin
    Pin oldpin=(Pin)pins.get(rempot);
    Pin newpin=(Pin)pins.get(newpot);

// set the components using oldpin to newpin
    if(oldpin.getComponentCount()>0) {
      throw new InternalError();
    }

    if(rempot+1!=pins.size()) { // move pin position forward
      throw new InternalError();
    }
    pins.remove(pins.size()-1);

    return(newpot);
  }




// ************************************************************************* //
//
//			Connectpoint management
//
// ************************************************************************* //


/** Add a connection point at x, y
 *  @returns the index of the new connectpoint
 */
  private int addConnectPoint(int x, int y, int pot) {
    //TRACE (".addConnectPoint(x="+x+", y="+y+", pot="+pot+").");

    int pp;
    int offs=0;

    for(pp=0; pp<connectcnt; pp++) {
      offs=pp*3;
      if(connectpoints[offs]>x) break;
      if((connectpoints[offs]==x)&&(connectpoints[offs+1]>=y)) break;
    }

    if((connectpoints[offs]==x)&&(connectpoints[offs+1]==y)) {
      throw new IllegalArgumentException("Duplicates");
    }

    if(connectcnt*3==connectpoints.length) {
      throw new InternalError();
    }

    offs=pp*3;
    if(pp<connectcnt) {
      System.arraycopy(connectpoints, offs, connectpoints, offs+3,
                       (connectcnt-pp)*3);
    }

    connectpoints[offs]=x;
    connectpoints[offs+1]=y;
    connectpoints[offs+2]=pot;
    connectcnt++;

    //ARM ("#: added connectpt at "+pp+", count "+connectcnt);
    return(pp);
  }




  private void removeConnectPoint(int x, int y) {
    //TRACE (".removeConnectPoint(x="+x+", y="+y+").");

    for(int pp=0; pp<connectcnt; pp++) {
      int offs=pp*3;

      if(connectpoints[offs]>x) break;
      if((connectpoints[offs]==x)&&(connectpoints[offs+1]==y)) {
        connectcnt--;
        if(pp!=connectcnt) {
          System.arraycopy(connectpoints, offs+3, connectpoints, offs,
                           (connectcnt-pp)*3);
        }
        return;
      }
    }

    throw new IllegalArgumentException();
  }





// ************************************************************************* //
//
//			Component management
//   query/add/modify
//
// ************************************************************************* //


  private synchronized int getComponentIndexAtGridPos(int x, int y) {
    for(int i=0; i<componentcnt; i++) {
      if((positions[i<<2]<=x)&&(positions[(i<<2)+1]<=y)&&
         ((positions[i<<2]+positions[(i<<2)+2])>x)&&
         ((positions[(i<<2)+1]+positions[(i<<2)+3])>y)) return(i);
    }
    return(-1);
  }


  private synchronized int getComponentIndexAtPoint(int x, int y) {
    x=(x+start.x)>>scale;
    y=(y+start.y)>>scale;

    for(int i=0; i<componentcnt; i++) {
      if((positions[i<<2]<=x)&&(positions[(i<<2)+1]<=y)&&
         ((positions[i<<2]+positions[(i<<2)+2])>x)&&
         ((positions[(i<<2)+1]+positions[(i<<2)+3])>y)) return(i);
    }
    return(-1);
  }








/** Add the component at given place to the simulation. If
 *  the component is at a visible region it will be
 *  assigned a graphics context and displayed
 *  @param x, y the position coords in the grid coord system
 *  @returns true if the component intersects with another
 */
  private synchronized boolean addComponent(Component c, int x, int y) {
    logger.fine(".addComponent(component="+c.getName()+", x="+x+", y="+y+").");

    java.awt.Dimension	s;

    int pos;
    for(pos=0; pos<componentcnt; pos++) {
      if(positions[pos<<2]>x) break;
    }

    if(componentcnt==components.length) {
      setComponentsArrayLength(componentcnt+(componentcnt>>2)+16);
    }

    if(pos<componentcnt) {
      System.arraycopy(components, pos, components, pos+1, componentcnt-pos);
      System.arraycopy(positions, pos<<2, positions, (pos+1)<<2,
                                  (componentcnt-pos)<<2);
    }

    s=c.getSize();

    logger.finer("Added component has size "+s);

    positions[pos<<2]=x;
    positions[(pos<<2)+1]=y;
    positions[(pos<<2)+2]=s.width;
    positions[(pos<<2)+3]=s.height;
    components[pos]=c;

    timer.addComponent(c);

    componentcnt++;

// connect the component if possible
    connectComponent(pos);

    int offx=(x<<scale)+start.x;
    int offy=(y<<scale)+start.y;
    int pwidth=s.width<<scale;
    int pheight=s.height<<scale;

    if(((offx+pwidth)<=0)||((offy+pheight)<=0)||
       (offx>=size.width)||(offy>=size.height)) {
      //ARM ("#: added component currently invisible.");
      return(false);
    }

// get the graphics context
    c.setGUIParameters(new Point(offx, offy), scale, c.getOrientation());

    return(false);
  }




/** Get the pin at the pos x,y of any component.
 *  @returns the number of the component and the num of the
 *  pin in an array. If multiple pins are located at x,y the
 *  array will contain 2*n valus
 */
  private int[] getComponentPinsAt(int x, int y) {

    for(int i=0; i<componentcnt; i++) {
      throw new InternalError();
    }

    return(null);
  }


/** Disconnect all pins of a given component.
 */
  private synchronized void disconnectComponent(int num) {

    for(int pc=0; pc<components[num].getPinCount(); pc++) {
      components[num].setPin(pc, null);
    }
  }

/** Connect all pins of a given component. If a componentpin
 *  was already connected, the method check if the
 *  connections were correct.
 */
  private synchronized void connectComponent(int num) {
    int x=positions[num<<2];
    int y=positions[(num<<2)+1];

    for(int pc=0; pc<components[num].getPinCount(); pc++) {
      Point ppos=components[num].getPinPos(pc);

      int pot=getPotentialAtGridPos(x+ppos.x, y+ppos.y);

      if(pot>=0) {
        Pin oldpin=components[num].getPin(pc);
        Pin newpin=(Pin)pins.get(pot);
        if((oldpin!=null)&&(oldpin!=newpin)) {
          logger.severe("Pin missmatch for component "+num+", pin "+pc);
        }

        pot=connectLinesAt(x+ppos.x, y+ppos.y);
        components[num].setPin(pc, newpin);
      } else if(components[num].getPin(pc)!=null) {
        logger.severe("Pin "+pc+" for component "+num+
                      " is connected, but shouldn't be");
      }
    }
  }



/** Connect component pins to underlying lines. If the
 *  component's pin is over an unconnected crossing of two
 *  lines, they will be connected together. That's why the
 *  connectComponents method should only be called when line
 *  array is fully consistent.<Br>
 *  This method should be used to connect existing components
 *  to a new line (fast), but not to connect a new component.
 */
  private synchronized void connectComponentsToLine(int linenum) {
    //TRACE (".connectComponentsToLine(linenum="+linenum+").");

    int offs=linenum*5;
    int fx=lines[offs];     int fy=lines[offs+1];
    int tx=lines[offs+2];   int ty=lines[offs+3];
    int pot=lines[offs+4];

    for(int i=0; i<componentcnt; i++) {
      offs=i<<2;
      if((positions[offs]>tx)||(positions[offs]+positions[offs+2]<fx)) continue;

// loop over all pins
      //ARM ("#: checking component "+components[i]);
      for(int ppos=0; ppos<components[i].getPinCount(); ppos++) {
        Point pos=components[i].getPinPos(ppos);
        pos.x+=positions[offs];
        pos.y+=positions[offs+1];

        if(isPointOnLine(pos.x, pos.y, linenum)) {
          pot=connectLinesAt(pos.x, pos.y);
          if(components[i].getPin(ppos)==null) {
            components[i].setPin(ppos, (Pin)(pins.get(pot)));
          }
          if(components[i].getPin(ppos)!=pins.get(pot)) {
            throw new InternalError();
          }
        }
      }
    }
    //ARM (".connectComponentsToLine() complete");
  }



// ************************************************************************* //
//
//			IO-Glue
//
// ************************************************************************* //


  private final static byte HEX_BYTES[]=
                     {'0', '1', '2', '3', '4', '5', '6', '7',
                      '8', '9', 'A', 'B', 'C', 'D', 'E', 'F'};


  public void write(fips.util.Url url) throws java.io.IOException {
    java.io.OutputStream out=url.getOutputStream();

    timer.stopTimer();

    out.write("#\n# DigSim V@version@ circuit file\n#\n\n".getBytes());

    StringBuffer sbuf=new StringBuffer();

    out.write(("lines="+linecnt+"\n").getBytes());
    for(int i=0; i<linecnt; i++) {
      int offs=i*5;
      sbuf.setLength(0);
      sbuf.append(lines[offs]).append(',').append(lines[offs+1]).append(',').
           append(lines[offs+2]).append(',').append(lines[offs+3]).
           append(',').append(lines[offs+4]).append('\n');
      out.write(sbuf.toString().getBytes());
    }
    out.write('\n');

    out.write(("connectpoints="+connectcnt+"\n").getBytes());
    for(int i=0; i<connectcnt; i++) {
      int offs=i*3;
      sbuf.setLength(0);
      sbuf.append(connectpoints[offs]).append(',').
           append(connectpoints[offs+1]).append(',').
           append(connectpoints[offs+2]).append('\n');
      out.write(sbuf.toString().getBytes());
    }
    out.write('\n');


    java.io.ByteArrayOutputStream bos=new java.io.ByteArrayOutputStream();

    out.write(("components="+componentcnt+"\n").getBytes());
    for(int i=0; i<componentcnt; i++) {
      int offs=i<<2;

// class,name,x,y,rot
      sbuf.setLength(0);
      sbuf.append(components[i].getClass().getName()).append(',').
           append(components[i].getName()).append(',').
           append(positions[offs]).append(',').append(positions[offs+1]).
           append(',').append(components[i].getOrientation()).append(',');
      out.write(sbuf.toString().getBytes());

      bos.reset();
      java.io.ObjectOutputStream oos=new java.io.ObjectOutputStream(bos);
      components[i].writeState(oos);
      oos.flush();
      oos.close();

      byte buf[]=bos.toByteArray();
      for(int bc=0; bc<buf.length; bc++) {
        out.write(HEX_BYTES[(buf[bc]&0xFF)>>4]);
        out.write(HEX_BYTES[buf[bc]&0xF]);
      }
      out.write('\n');
    }
    out.write('\n');
  }


  public final static int LINES_SECTION		= 0;
  public final static int CONNECTPTS_SECTION	= 1;
  public final static int COMPONENTS_SECTION	= 2;


/** Read the circuit description from file.
 *  The current circuit in this canvas is dropped.
 *  The class definitions for all included components have
 *  to be available in the current Editor.
 */
  public void read(fips.util.Url url) throws java.io.IOException {
    java.io.InputStream in=url.getInputStream();

    timer.stopTimer();
    deleteCircuit();

    java.io.LineNumberReader lnr;
    lnr=new java.io.LineNumberReader(new java.io.InputStreamReader(in));

    int section=-1;
    String line;
    while((line=lnr.readLine())!=null) {
      line=line.trim();
      if((line.length()==0)||(line.charAt(0)=='#')) continue;

      try {

        if(line.indexOf('=')!=-1) { // new section command
          if(line.startsWith("lines=")) {
            section=LINES_SECTION;
            int i=Integer.parseInt(line.substring(6));
            setLineArrayLength(i+(i>>4)+20);
            linecnt=0;
          } else if(line.startsWith("connectpoints=")) {
            section=CONNECTPTS_SECTION;
            int i=Integer.parseInt(line.substring(14));
            setConnectPointsArrayLength(i+(i>>4)+20);
            connectcnt=0;
          } else if(line.startsWith("components=")) {
            section=COMPONENTS_SECTION;
            int i=Integer.parseInt(line.substring(11));
            setComponentsArrayLength(i+(i>>4)+20);
          } else {
            throw new IllegalArgumentException("Unknown section command "+line);
          }
          continue;
        } 

        int offs;
        int pos;
        switch(section) {
          case LINES_SECTION:
            offs=linecnt*5;
            pos=0;
            for(int i=0; i<5; i++) {
              int next=line.indexOf(',', pos);
              if(next<0) next=line.length();
              lines[offs+i]=Integer.parseInt(line.substring(pos, next));
              pos=next+1;
            }
// add the pins if necessary
            int pot=lines[offs+4];

            if(pins.size()==pot) pins.add(new Pin());
            else if(pins.size()<pot) {
              while(pins.size()<pot) pins.add(null);
              pins.add(new Pin());
            } else {
              if(pins.get(pot)==null) pins.set(pot, new Pin());
            }

            linecnt++;
            break;
          case CONNECTPTS_SECTION:
            offs=connectcnt*3;
            pos=0;
            for(int i=0; i<3; i++) {
              int next=line.indexOf(',', pos);
              if(next<0) next=line.length();
              connectpoints[offs+i]=Integer.parseInt(line.substring(pos, next));
              pos=next+1;
            }
            connectcnt++;
            break;
          case COMPONENTS_SECTION:
// fips.sim.dig.comp.Button,Button,1,1,0,03
            pos=line.indexOf(',');
            Component comp=null;
            comp=(Component)getClass().
                         getClassLoader().loadClass(line.substring(0, pos)).
                         newInstance();
            pos++;
            comp.setName(line.substring(pos, line.indexOf(',', pos)));
            pos=line.indexOf(',', pos)+1;

// now read x, y, orientation, config data
            int ival[]=new int[3];
            for(int i=0; i<3; i++) {
              int next=line.indexOf(',', pos);
              ival[i]=Integer.parseInt(line.substring(pos, next));
              pos=next+1;
            }

// now read the hex data
            java.io.ByteArrayOutputStream bos;
            bos = new java.io.ByteArrayOutputStream();

            while(pos<line.length()) {
              char c=line.charAt(pos++);
              if(c!=' ') {
                int i=Character.digit(c, 16);
                c=line.charAt(pos++);
                if(c!=' ') {
                  i=(i<<4)|Character.digit(c, 16);
                }
                if(i<0) throw new IllegalArgumentException();
                bos.write(i);
              }
            }

// init the component
            if(bos.size()>0) {
              java.io.ByteArrayInputStream bis=new java.io.ByteArrayInputStream(bos.toByteArray());
              comp.readState(new java.io.ObjectInputStream(bis));

              if(bis.available()>0) throw new InternalError();
            }
            comp.setGUIParameters(new Point(ival[0], ival[1]),
                                  0, ival[2]);

// now add the component
            addComponent(comp, ival[0], ival[1]);

            break;
          default:
            throw new InternalError("Section \""+section+"\" in line "+
                                    lnr.getLineNumber()+" not implemented.");
        }


/*
    out.write(("connectpoints="+connectcnt+"\n").getBytes());
    for(int i=0; i<connectcnt; i++) {
      int offs=i*3;
      sbuf.setLength(0);
      sbuf.append(connectpoints[offs]).append(',').
           append(connectpoints[offs+1]).append(',').
           append(connectpoints[offs+2]).append('\n');
      out.write(sbuf.toString().getBytes());
    }
    out.write('\n');
*/

      } catch (Exception e) {
        StringBuffer sbuf=new StringBuffer();
        sbuf.append("Read error in line ").append(lnr.getLineNumber()).
             append(", line=\"").append(line).append("\", error=");

        if((e.getMessage()==null)||(e.getMessage().length()==0)) {
          sbuf.append(e);
        } else sbuf.append(e.getMessage());
        IllegalArgumentException iae;
        iae=new IllegalArgumentException(sbuf.toString());
        iae.setStackTrace(e.getStackTrace());
        throw(iae);
      }
    }

    paintCircuitCanvas(getGraphics());

    logger.finer("Read complete.");
  }









//**************************************************************************//
//
//				Cursor support				    //
//
//**************************************************************************//


  private void showGateCursor(Point coord) {


    Point pos=null;
    if(coord!=null) {

      pos=getNearestGridPoint(coord.x-(currgate.getSize().width<<(scale-1)),
                              coord.y-(currgate.getSize().height<<(scale-1)));

    }

    logger.finest("#: new gatecursorpos is "+pos);

    if((gateposition!=null)&&(!gateposition.equals(pos))) {
// remove old cursor image

/*
      repaintCircuitCanvas(start.x+(gateposition.x<<scale),
                           start.y+(gateposition.y<<scale),
                           gatecursor.getWidth(null),
                           gatecursor.getHeight(null));
*/


      repaintCircuitCanvas(start.x+(gateposition.x<<scale),
                          start.y+(gateposition.y<<scale),
                          gatecursor.getWidth(null),
                          gatecursor.getHeight(null));
/*
      repaint(start.x+(gateposition.x<<scale), start.y+(gateposition.y<<scale),
              gatecursor.getWidth(null), gatecursor.getHeight(null));
*/
      gateposition=null;

    }

    if(pos==null) return;
    if(pos.equals(gateposition)) return;

// draw gate cursor
    getGraphics().drawImage(gatecursor, (pos.x<<scale)+start.x,
                            (pos.y<<scale)+start.x , this);
    

    gateposition=pos;

  }


  public JToolTip createToolTip() {
    logger.fine(".createToolTip()");

    return(super.createToolTip());
  }


  public Point getToolTipLocation(MouseEvent event) {
    logger.fine(".getToolTipLocation(event).");

    return(event.getPoint());
  }

  public String getToolTipText(MouseEvent me) {
    logger.fine(".getToolTipText(me)");

    int cpos=getComponentIndexAtPoint(me.getX(), me.getY());
    return(cpos<0?null:components[cpos].getName());
  }


// ************************************************************************* //
//
//			Paint method implementaion			     //
//
// ************************************************************************* //


/** From JComponent
 */

  protected void paintComponent(Graphics g) {
    logger.fine(".paintComponent()");

//    super.paintComponent(g);
//    paintCircuitCanvas(getGraphics());
  }


  public void paint(Graphics g) {
    paintCircuitCanvas(g);
  }

  public void repaintCircuitCanvas() {
    repaintCircuitCanvas(0, 0, getWidth(), getHeight());
  }

  private void repaintCircuitCanvas(int x, int y, int w, int h) {
    //TRACE ("#: repaintCircuitCanvas");

    repaint(x,y,w,h);
  }

/** Repaint region with coords from grid coord system
 */
  public void repaintGrid(int x, int y, int w, int h) {
    repaint((x<<scale)-start.x, (y<<scale)-start.y, w<<scale, h<<scale);
  }




/** Full repaint of canvas.
 */
  protected void paintCircuitCanvas(Graphics g) {
    logger.fine("paintCircuitCanvas()");

    logger.fine(".paint(g="+g.hashCode()+"): clip="+g.getClip());

    int unit=1<<scale;

    java.awt.Rectangle clip=g.getClipBounds();
    if(clip==null) clip=new java.awt.Rectangle(0, 0, getWidth(), getHeight());

    g.setColor(Color.black);
    g.fillRect(clip.x, clip.y, clip.width, clip.height);

    java.awt.Rectangle gridclip=new java.awt.Rectangle();
    gridclip.x=(clip.x+start.x)>>scale;
    gridclip.y=(clip.y+start.y)>>scale;

    int x=clip.x+clip.width+start.x;
    gridclip.width=(((x&(-1<<scale))==x)?x>>scale:(x>>scale)+1)-gridclip.x;

    int y=clip.y+clip.height+start.y;
    gridclip.height=(((y&(-1<<scale))==y)?y>>scale:(y>>scale)+1)-gridclip.y;

    int offx=(gridclip.x<<scale)-start.x;
    int offy=(gridclip.y<<scale)-start.y;

    logger.finer("Repainting circuit from "+gridclip.x+"x"+gridclip.y+
                 ", width "+gridclip.width+"x"+gridclip.height);


// repaint the grid

    g.setColor(Color.green);

    for(x=0; x<gridclip.width; x++) {
      int posx=offx+(x<<scale);

      for(y=0; y<gridclip.height; y++) {
        int posy=offy+(y<<scale);
        g.drawLine(posx, posy, posx, posy);
      }
    }

    logger.finer("Painted grid");

// repaint the components

    int endx=gridclip.x+gridclip.width;
    int endy=gridclip.y+gridclip.height;
    logger.finer("Looking for components from "+gridclip.x+"x"+gridclip.y+
                 " to "+endx+"x"+endy);

    for(int cpos=0; cpos<componentcnt; cpos++) {
      logger.finer("Redraw comp "+cpos+", from "+positions[cpos<<2]+"x"+
                   positions[(cpos<<2)+1]+", dimension ");

      if(positions[cpos<<2]>=endx) break;

      if((positions[(cpos<<2)+1]>=endy)||
         (positions[cpos<<2]+positions[(cpos<<2)+2]<=gridclip.x)||
         (positions[(cpos<<2)+1]+positions[(cpos<<2)+3]<=gridclip.y)) continue;

      logger.finest("Found component "+components[cpos].getName()+
                    " to repaint");
      components[cpos].repaint(g);
    }

    logger.finer("Painted components");

// repaint the lines
    g.setColor(Color.green);
    for(int lpos=0; lpos<linecnt; lpos++) {
      int offs=lpos*5;

      if((lines[offs]>=endx)||(lines[offs+2]<gridclip.x)) break;

//    g.setColor((lines[offs+4]&1)==0?Color.green:Color.red);
      g.setColor(Color.green);

      g.drawLine((lines[offs]<<scale)-start.x, (lines[offs+1]<<scale)-start.y,
              (lines[offs+2]<<scale)-start.x, (lines[offs+3]<<scale)-start.y);

    }

// repaint the connectors (if any)
    for(int cpos=0; cpos<connectcnt; cpos++) {
      int offs=cpos*3;

      //NEVER ("#: painting point at "+connectpoints[offs]+"x"+
      //NEVER  connectpoints[offs+1]);

      if(connectpoints[offs]<gridclip.x) continue;
      if(connectpoints[offs]>=endx) break;

      if((connectpoints[offs+1]>=gridclip.y)&&
         (connectpoints[offs+1]<endy)) {
        g.fillOval((connectpoints[offs]<<scale)-start.x-2,
                   (connectpoints[offs+1]<<scale)-start.y-2, 4, 4);
      }
    }
  }







//**************************************************************************//
//
//			Implementation of TimerListener
//
//********************


  public void timerStarted(BigInteger time) {}
  public void timerStopped(BigInteger time) {}
  public void timerStepComplete(BigInteger time) {
    for(int i=0; i<componentcnt; i++) {
      if(components[i].needsUpdate()) {
        logger.finer("Component "+components[i].getName()+" needs update");
        repaint();
        break;
      }
    }
  }



}






