
package fips.sim.dig.edit;

import java.io.IOException;
import java.util.Hashtable;
import java.util.logging.Handler;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JFileChooser;
import javax.swing.JMenuItem;


public class Editor implements java.awt.event.ActionListener,
                               java.awt.event.ComponentListener {

  public static void main(String args[]) {
    Editor edit = new Editor();

    if(args.length!=0) {
      if(args[0].indexOf("://")<0) args[0]="file://"+args[0];
      try {
        edit.loadCircuit(new fips.util.Url(args[0]));
      } catch (IOException ioe) {
        ioe.printStackTrace();
      }
    }
  }


  Logger				editlog;

/** Systemwide rc-file.
 */
  private RCFile			rcfile;


  private ParameterGroup		pgroup;


/** Main frame
 */
  javax.swing.JFrame			frame;

  Hashtable				menuitems;


  java.awt.Rectangle			bounds;

  GateSelect				gateselect;

  CircuitCanvas				circuit;

/** Last dir of Filechooser
 */
  java.io.File				lastselecteddir;




  fips.util.Url				circuiturl;



  public Editor() {

// setup the logging first
    Logger logger = Logger.getLogger("digsim");
    logger.setLevel(Level.INFO);

    Handler lh[]=logger.getHandlers();
    for(int i=0; i<lh.length; i++) {    
      logger.removeHandler(lh[i]);
    }

    Handler ch = new java.util.logging.ConsoleHandler();
    ch.setLevel(Level.ALL);
    ch.setFormatter(new java.util.logging.SimpleFormatter());
    logger.addHandler(ch); 

    editlog = Logger.getLogger("digsim.edit");
    try {
      editlog.setParent(logger);
    } catch (Exception e) { // ignore security exceptions in javaws
      editlog.severe("Can't set parent of editlogger.");
      editlog.log(Level.FINE, "Caught unexpected exception", e);
    }

//    Logger.getLogger("digsim.edit").setLevel(Level.ALL);
    Logger.getLogger("digsim.edit").setLevel(Level.INFO);
    Logger.getLogger("digsim.edit.configure").setLevel(Level.INFO);
    Logger.getLogger("digsim.comp").setLevel(Level.INFO);


    String sysrcfileurl=getSystemProperty("rcfile", "sys:///digsimconfig.xml");
    String userrcfileurl=getSystemProperty("userrcfile", null);

 
    if(sysrcfileurl!=null) {
      try {
        editlog.info("Loading system config file "+sysrcfileurl);

// open system rc-file in non-write mode
        rcfile = new RCFile(false, sysrcfileurl, null);
      } catch (IOException ioe) {
        editlog.log(Level.SEVERE, "Can't access rcfile-url \""+sysrcfileurl+
                                  "\"", ioe);
      } catch (IllegalArgumentException iea) {
        iea.printStackTrace();
      }
    }
 


// now load the user rcfile
    if((userrcfileurl==null)||(userrcfileurl.equals("none"))) {
// create a dummy rcfile (rw)
// exception should be impossible

      try {
        if(rcfile==null) rcfile=new RCFile(true, null, null);
        else rcfile=new RCFile(true, null, rcfile);
      } catch (Exception e) {
        editlog.log(Level.SEVERE, "Failed to create dummy rcfile", e);
        System.exit(1);
      }
    } else {
      editlog.info("Reading user rcfile "+userrcfileurl);


      try {
        rcfile=new RCFile(true, userrcfileurl, rcfile);
      } catch (Exception e) {
        editlog.log(Level.SEVERE, "Can't access rcfile-url \""+userrcfileurl+
                                  "\", no persisten storage of user settings", e);
        try {
          rcfile=new RCFile(true, null, rcfile);
        } catch (Exception e2) {
          editlog.log(Level.SEVERE, "Can't open default mem-rc", e2);
          System.exit(1);
        }
      }
    }


    pgroup=rcfile.getParameterGroup(null).getGroup("editor");
    if(pgroup==null) {
      rcfile.getParameterGroup(null).createGroup("editor");
      pgroup=rcfile.getParameterGroup(null).getGroup("editor");
    }


    frame = new javax.swing.JFrame();
    frame.setTitle("Digital Circuit Simulator V@version@");
    menuitems=new Hashtable();

    javax.swing.JMenuBar mb=new javax.swing.JMenuBar();

    javax.swing.JMenu men=new javax.swing.JMenu("File");
    men.add(createMenuItem("Load", "file_load", true));
    men.add(createMenuItem("Save", "file_save", true));
    men.add(createMenuItem("Save As", "file_saveas", true));
    men.addSeparator();
    men.add(createMenuItem("Exit", "file_exit", true));
    mb.add(men);


    men=new javax.swing.JMenu("Simulation");
    men.add(createMenuItem("Start", "sim_start", true));
    men.add(createMenuItem("Stop", "sim_stop", false));
    men.add(createMenuItem("Step", "sim_step", true));
    mb.add(men);

    frame.setJMenuBar(mb);


    circuit=new CircuitCanvas(); 
//    circuit.setDoubleBuffered(false);

    frame.setContentPane(circuit);
//    frame.getContentPane().add(circuit);

    bounds=pgroup.getBounds("window");
    if(bounds==null) bounds=new java.awt.Rectangle(0, 0, 300, 250);
 
//    frame.pack();
 
    frame.setBounds(bounds);
    frame.setVisible(true);

    frame.addComponentListener(this);

    gateselect = new GateSelect(rcfile);
    gateselect.addSelectionListener(circuit);
  }





  private JMenuItem createMenuItem(String name, String command,
                                   boolean enabled) {
    editlog.finer(".createMenuItem(name=\""+name+"\", command=\""+
                  command+"\", enabled="+enabled+").");

    JMenuItem mi=new JMenuItem(name);
    mi.setActionCommand(command);
    mi.setEnabled(enabled);
    mi.addActionListener(this);
    menuitems.put(command, mi);
    return(mi);
  }


  private String getSystemProperty(String param, String defvalue) {
    String ret=null;

    try {
      ret=System.getProperty(param);
    } catch (Exception e) {
      editlog.warning("Fetch of system-property \""+param+"\" failed");
      editlog.log(Level.FINE, "Caught exception: ", e);
    }
    if(ret==null) ret=defvalue;
    return(ret);
  }



//******
//
//			Actionlistener impl
//
//******

  public void actionPerformed(java.awt.event.ActionEvent ae) {
    String cmd=ae.getActionCommand();
    editlog.finer(".actionPerformed(cmd="+cmd+").");

    if(cmd.startsWith("file_")) {
      cmd=cmd.substring(5);

      if((cmd.equals("save"))||(cmd.equals("saveas"))) {

        fips.util.Url url=null;

        if((cmd.equals("save"))&&(circuiturl!=null)) url=circuiturl;
        else {
          JFileChooser fc;
          java.io.File file=null;
          fc=new JFileChooser(file);

          if(lastselecteddir!=null) fc.setCurrentDirectory(lastselecteddir);
          if(fc.showSaveDialog(null)==fc.APPROVE_OPTION) {
            file=fc.getSelectedFile();
          }
          lastselecteddir=fc.getCurrentDirectory();

          if(file==null) return;

          url=new fips.util.Url("file://"+file.getPath());
        }


// write the circuit
        try {
          circuit.write(url);
          circuiturl=url;
        } catch (IOException ioe) {
          ioe.printStackTrace();
        }

        return;
      }
      

      if(cmd.equals("load")) {
        JFileChooser fc;
        java.io.File file=null;

        fc=new JFileChooser();
        if(lastselecteddir!=null) fc.setCurrentDirectory(lastselecteddir);
        if(fc.showOpenDialog(null)==fc.APPROVE_OPTION) {
          file=fc.getSelectedFile();
        }

        lastselecteddir=fc.getCurrentDirectory();
        if(file==null) return;

// read the circuit
        try {
          loadCircuit(new fips.util.Url("file://"+file.getPath()));
        } catch (IOException ioe) {
          ioe.printStackTrace();
        }

        return;
      }

      if(cmd.equals("exit")) {
        editlog.severe("No exitcheck yet!!!");
        System.exit(0);
      }

      throw new IllegalArgumentException("Illegal command \""+cmd+"\"");
    }

    if(cmd.startsWith("sim_")) {
      cmd=cmd.substring(4);

      if(cmd.equals("start")) {
        circuit.timer.startTimer(null);
        ((JMenuItem)menuitems.get("sim_start")).setEnabled(false);
        ((JMenuItem)menuitems.get("sim_stop")).setEnabled(true);
        return;
      }

      if(cmd.equals("stop")) {
        circuit.timer.stopTimer();
        ((JMenuItem)menuitems.get("sim_stop")).setEnabled(false);
        ((JMenuItem)menuitems.get("sim_start")).setEnabled(true);
        return;
      }

      if(cmd.equals("step")) {
        circuit.timer.doStep();
        return;
      }

      throw new IllegalArgumentException("Illegal command \""+cmd+"\"");
    }

    throw new IllegalArgumentException("Illegal command \""+cmd+"\"");
  }




//**************************************************************************//
//
//			ComponentListener implementation		    //
//
//**************************************************************************//


  public void componentMoved(java.awt.event.ComponentEvent ce) {
    editlog.finer(".componentMoved(ce)");
 
    if((frame.getX()!=bounds.x)||(frame.getY()!=bounds.y)) {
      bounds.x=frame.getX();
      bounds.y=frame.getY();
      pgroup.setBounds("window", bounds);
    }
  }
  public void componentResized(java.awt.event.ComponentEvent ce) {
    //editlog.finer(".componentResized(ce)");
 
    if((frame.getWidth()!=bounds.width)||(frame.getHeight()!=bounds.height)) {
      bounds.width=frame.getWidth();
      bounds.height=frame.getHeight();
      pgroup.setBounds("window", bounds);
    }
  }


 
  public void componentShown(java.awt.event.ComponentEvent ce) {
    //TRACE (".componentShown(ce)");
  }
 
  public void componentHidden(java.awt.event.ComponentEvent ce) {
    //TRACE (".componentHidden(ce)");
  }





//**************************************************************************//
//
//			Private glue
//
//**************************************************************************//


  protected void loadCircuit(fips.util.Url url) throws IOException {
    circuit.read(url);
    circuiturl=url;
    ((JMenuItem)menuitems.get("file_save")).setEnabled(url.isWritable());
  }

}

















