package fips.sim.dig.edit.configure;

import java.awt.event.ActionListener;
import java.util.logging.Logger;

import javax.swing.JComponent;
import javax.swing.JLabel;

import fips.sim.dig.base.Component;
import fips.sim.dig.base.Parameter;
import fips.sim.dig.base.ParameterChangeListener;
import fips.sim.dig.base.StringParameter;


/** This class configures the hardware components components.
 *  The configurator displays the description of the component
 *  

 */
public class ComponentConfigurator implements ActionListener,
                                              ParameterChangeListener {

  private Logger		logger;

  javax.swing.JDialog		dialog;

  JLabel			header;
  javax.swing.JTextArea		description;
  javax.swing.JScrollPane	scroll;

  Parameter			parameters[];
  EditableParameter		parameditors[];


  Component			hwcomp;

/** Flag to indicate that one or more parameters have changed
 */
  boolean			changeflag;

/** List of all parameter values when configurator was
 *  started.
 */
  Object			oldvalues[];

  public ComponentConfigurator() {

    logger=Logger.getLogger("digsim.edit.configure");

    dialog=new javax.swing.JDialog();
    dialog.setTitle("Change configuration");
    dialog.setModal(true);

// create the gui
    java.awt.GridBagLayout gbl=new java.awt.GridBagLayout();
    dialog.getContentPane().setLayout(gbl);

    java.awt.GridBagConstraints gbc = new java.awt.GridBagConstraints();

    gbc.gridwidth=3;
    gbc.gridx=0;
    gbc.gridy=0;
    gbc.weightx=1.0;
    gbc.weighty=0.0;
    gbc.fill=gbc.NONE;
    gbc.insets=new java.awt.Insets(16, 1, 16, 1);

    gbl.setConstraints(header=new JLabel(), gbc);
    dialog.getContentPane().add(header);

    gbc.gridy++;
    gbc.insets=new java.awt.Insets(4, 4, 4, 4);
    JLabel label;
    gbl.setConstraints(label=new JLabel("Description"), gbc);
    dialog.getContentPane().add(label);

    gbc.gridy++;
    gbc.fill=gbc.BOTH;
    gbc.weighty=0.2;
    description=new javax.swing.JTextArea();
    description.setEditable(false);
    description.setRows(5);

    javax.swing.JScrollPane dscroll=new javax.swing.JScrollPane();
    dscroll.setHorizontalScrollBarPolicy(scroll.HORIZONTAL_SCROLLBAR_NEVER);
    dscroll.setVerticalScrollBarPolicy(scroll.VERTICAL_SCROLLBAR_AS_NEEDED);
    dscroll.setViewportView(description);

    gbl.setConstraints(dscroll, gbc);
    dialog.getContentPane().add(dscroll);

    gbc.gridy++;
    gbc.weighty=0.0;
    gbl.setConstraints(label=new JLabel("Parameters"), gbc);
    dialog.getContentPane().add(label);

    gbc.gridy++;
    gbc.anchor=gbc.NORTH;
    gbc.fill=gbc.BOTH;
    gbc.weighty=1.0;
    scroll=new javax.swing.JScrollPane();
    scroll.setHorizontalScrollBarPolicy(scroll.HORIZONTAL_SCROLLBAR_NEVER);
    scroll.setVerticalScrollBarPolicy(scroll.VERTICAL_SCROLLBAR_AS_NEEDED);
    gbl.setConstraints(scroll, gbc);
    dialog.getContentPane().add(scroll);


    // now add the buttons

    javax.swing.JPanel pan = new javax.swing.JPanel();
    pan.setLayout(new java.awt.FlowLayout());

    javax.swing.JButton button=new javax.swing.JButton("Close");
    button.setActionCommand("close");
    button.addActionListener(this);
    pan.add(button);

    button=new javax.swing.JButton("Cancel");
    button.setActionCommand("cancel");
    button.addActionListener(this);
    pan.add(button);

    button=new javax.swing.JButton("Reset to Defaults");
    button.setActionCommand("reset");
    button.addActionListener(this);
    pan.add(button);


    gbc.gridy++;
    gbc.gridx=0;
    gbc.gridwidth=3;
    gbc.weightx=1.0;
    gbc.weighty=0;
    gbl.setConstraints(pan, gbc);
    dialog.getContentPane().add(pan);
    
  }



/** @returns true if parameters were changed
 */
  public boolean configure(Component c) {
    synchronized(this) {
      if(hwcomp!=null) throw new IllegalStateException("Can't configure two components at the same time.");

      hwcomp=c;
    }

    header.setText(c.getComponentName());
    description.setText(c.getDescription());

    initParamList();

    dialog.pack();
    changeflag=false;
    dialog.setVisible(true);

    synchronized(this) {
      hwcomp=null;
    }

    return(changeflag);
  }



  private void initParamList() {
    javax.swing.JPanel panel = new javax.swing.JPanel();

    java.awt.GridBagLayout gbl=new java.awt.GridBagLayout();
    panel.setLayout(gbl);

    java.awt.GridBagConstraints gbc = new java.awt.GridBagConstraints();
    gbc.weighty=0.0;
    gbc.gridy=0;

    parameters=hwcomp.getParameters();
    oldvalues=new Object[hwcomp.getParameterCount()+1];
    parameditors=new EditableParameter[oldvalues.length];

    for(int i=0; i<parameditors.length; i++) {
      oldvalues[i]=i==0?hwcomp.getName():parameters[i-1].getValue();

      gbc.gridx=0;
      gbc.gridwidth=1;
      gbc.weightx=0.0;
      gbc.weighty=(i<oldvalues.length-1)?0:1;
      gbc.anchor=gbc.NORTH;
      gbc.fill=gbc.HORIZONTAL;
      gbc.insets=new java.awt.Insets(4, 20, 4, 4);

      JLabel label=new JLabel(i==0?"Name":parameters[i-1].getName());
      gbl.setConstraints(label, gbc);
      panel.add(label);

      gbc.gridx++;
      gbc.weightx=1.0;
      gbc.insets=new java.awt.Insets(4, 4, 4, 4);

      

      if(i==0) {
        Parameter param=new StringParameter("Name", "Name of the component",
                   hwcomp.getComponentName(), hwcomp.getName(), Integer.MAX_VALUE);
        parameditors[i]=new GenericStringParameterEditor(param);
      } else {
        logger.fine("Checking parameter "+parameters[i-1].getName());
        if(parameters[i-1] instanceof EditableParameter) {
          parameditors[i]=(EditableParameter)parameters[i-1];
        } else {
          if((parameters[i-1] instanceof StringParameter)||

             (parameters[i-1] instanceof fips.sim.dig.base.TimeParameter)) {
            parameditors[i]=new GenericStringParameterEditor(parameters[i-1]);
          } else if(parameters[i-1] instanceof fips.sim.dig.base.BooleanParameter) {
            parameditors[i]=new BooleanParameterEditor(parameters[i-1]);
          } else if(parameters[i-1] instanceof fips.sim.dig.base.ByteArrayParameter) {

            parameditors[i]=new ByteArrayParameterEditor(parameters[i-1]);
          } else {
            logger.severe("Parameter handling for type "+
                          parameters[i-1].getClass()+" not yet implemented");
            parameditors[i]=null;
          }
        }
      }

      JComponent comp=null;
      if(parameditors[i]==null) comp=new JLabel("Not supported");
      else {
        parameditors[i].addParameterChangeListener(this);
        comp=parameditors[i].getGUI();
      }

      gbl.setConstraints(comp, gbc);
      panel.add(comp);
      gbc.gridy++;
    }

    scroll.setViewportView(panel);
  }



  public void actionPerformed(java.awt.event.ActionEvent ae) {
    logger.fine(".actionPerformed(cmd="+ae.getActionCommand()+").");

    String cmd=ae.getActionCommand();

    if(cmd.equals("close")) {
      for(int i=0; i<parameditors.length; i++) {
        if(parameditors[i]==null) {
          logger.severe("Remove null workar.");
          continue;
        }
        if(parameditors[i].isEditing()) {
          logger.fine("Edit not complete, no close");
          return;
        }
      }
      dialog.setVisible(false);
      return;
    }

    if(cmd.equals("cancel")) {
      logger.severe("No parameter revert yet");

      // changeflag=false;
      dialog.setVisible(false);
      return;
    }

    if(cmd.equals("reset")) {
      logger.severe("Reset not yet implemented");
      return;
    }

    throw new InternalError("Unknown command \""+cmd+"\".");    
  }




  public void parameterChanged(Parameter param) {
    logger.fine(".parameterChanged(param="+param.getName()+").");

    if(param==parameditors[0]) {
      logger.finer("Setting component name to "+param.getValue());
      hwcomp.setName((String)param.getValue());
    } else changeflag=true;
  }



}




