package fips.sim.dig.edit.configure;

import java.util.ArrayList;
import java.util.logging.Logger;

import fips.sim.dig.base.AbstractParameter;

public abstract class EditableParameter extends AbstractParameter {

  private boolean	editmode;
  private ArrayList	listeners;

  protected EditableParameter(String name, String description,
                   Object defaultval, Object currentval,
                   boolean alterssize, boolean alterspinout,
                   boolean altersparams) {

    super(name, description, defaultval, currentval,
          alterssize, alterspinout, altersparams);

    listeners=new ArrayList();
    editmode=false;
  }

  public abstract javax.swing.JComponent getGUI();


/** Check if this gui is currently editing the parameter.
 */
  public synchronized boolean isEditing() {
    return(editmode);
  }


  public void addEditEventListener(EditEventListener eel) {
    synchronized(listeners) {
      if(eel!=null) listeners.add(eel);
    }
  }

  public void removeEditEventListener(EditEventListener eel) {
    synchronized(listeners) {
      listeners.remove(eel);
    }
  }


  protected void setEditing(boolean value) {
    synchronized(listeners) {
      if(editmode==value) return;

      editmode=value;
      for(int i=0; i<listeners.size(); i++) {
        if(value) ((EditEventListener)(listeners.get(i))).editStarted(this);
        else ((EditEventListener)(listeners.get(i))).editStopped(this);
      }
    }
  }

}


