package fips.sim.dig.edit.configure;

import java.awt.Color;

import java.util.logging.Logger;

import javax.swing.JComponent;
import javax.swing.JTextField;
import javax.swing.border.LineBorder;
import javax.swing.event.DocumentListener;

import fips.sim.dig.base.Parameter;
import fips.sim.dig.base.ParameterChangeListener;




/** This class describes a very simple EditableParameter
 *  implementation that allows the changing of parameters
 *  with simple text representation (single line)
 */
public class GenericStringParameterEditor extends AbstractWrappedParameterEditor
                                 implements java.awt.event.FocusListener,
                                            DocumentListener {
  private JTextField		component;

/** True if the current textfield value is valid.
 */
  boolean			contentvalid;

  public GenericStringParameterEditor(Parameter param) {
    super(param);
  }


  public synchronized JComponent getGUI() {
    if(component==null) {
      component=new javax.swing.JTextField();

      component.setColumns(30);
      component.setEditable(true);
      component.setHorizontalAlignment(component.LEADING);

      component.setText(param.toString());
      component.addFocusListener(this);
      component.getDocument().addDocumentListener(this);

      component.setBorder(new LineBorder(Color.black));
      contentvalid=true;
    }

    return(component);
  }





//**************************************************************


  public void focusGained(java.awt.event.FocusEvent fe) {
  }

  public synchronized void focusLost(java.awt.event.FocusEvent fe) {
    logger.fine(".focusLost()");
    if(!isEditing()) return;

    checkUpdate(true);
  }

  public void insertUpdate(javax.swing.event.DocumentEvent de) {
    checkUpdate(false);
  }

  public void removeUpdate(javax.swing.event.DocumentEvent de) {
    checkUpdate(false);
  }

  public void changedUpdate(javax.swing.event.DocumentEvent de) {
    checkUpdate(false);
  }

/** @param set if true and the component text valid, set the
 *  parameter
 */
  private synchronized void checkUpdate(boolean set) {
    logger.finer(".checkUpdate(set="+set+").");

    if(!isEditing()) {
      setEditing(true);
    }

    String str=component.getText();
    Object o=null;
    try {
      o=param.parseString(str);

      if(!contentvalid) {
        component.setBorder(new LineBorder(Color.black));
        contentvalid=true;
      }

      if(set) {
        logger.fine("Setting parameter value");
        setValue(o);
        setEditing(false);
      }
    } catch (IllegalArgumentException iae) {
      if(contentvalid) {
        component.setBorder(new LineBorder(Color.red));
        contentvalid=false;
      }
    }
  }


}




