package fips.util;


/** This class contains static methods for bitwise copying
 *  and moving of data in byte arrays.
 */
public class ByteArrayUtil {


  public final static byte hexchars[]={
             0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37,
             0x38, 0x39, 0x41, 0x42, 0x43, 0x44, 0x45, 0x46};

  public final static byte printmask[]={
     0,0,0,0,0,0,0,0,0,0,' ',0,0,0,0,0,
     0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
     ' ','!','"','#','$','%','&',0,'(',0,'*','+',0,'-','.','/',
     '0','1','2','3','4','5','6','7','8','9',':',0,'<','=','>','?',
     0,'A','B','C','D','E','F','G','H','I','J','K','L','M','N','O',
     'P','Q','R','S','T','U','V','W','X','Y','Z','[',0,']',0,0,
     0,'a','b','c','d','e','f','g','h','i','j','k','l','m','n','o',
     'p','q','r','s','t','u','v','w','x','y','z','{','|','}',0,0,
     0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
     0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
     0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
     0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
     0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
     0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
     0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
     0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
  };




/** Get one bit from bytearray.
 */
  public static boolean getBit(byte data[], int pos) {
    return((data[pos>>3]&(1<<(pos&7)))!=0);
  }


  public static void setBit(byte data[], int pos, boolean val) {
    data[pos>>3]&=~(1<<(pos&7));
    if(val) data[pos>>3]|=(1<<(pos&7));
  }






/** Compare parts of two byte arrays.
 *  @returns 0 if length bytes are the same, returns -k-1 where
 *  k is the position of the first difference if al[apos+k]<bl[bpos+k]
 *  or k+1 if bl > al.
 */
  public static int compare(byte al[], int apos, byte bl[], int bpos,
                            int length) {
    for(int i=0; i<length; i++) {
      if(al[apos+i]==bl[bpos+i]) continue;

      if(al[apos+i]<bl[bpos+i]) return(-i-1);
      return(i+1);
    }
    return(0);
  }




/** Dump byte buffer data to an output stream. 
 *  @param start position of the first byte to dump
 *  @param full if true produce full dump with translated chars
 */
  public static void hexDump(byte data[], int start, int length,
                             java.io.OutputStream out, boolean full)
                             throws java.io.IOException {
    byte buf[]=new byte[80];


    for(int i=0; i<length; i+=16) {
      int outpos=0;
      int bpos=0;

      while((i+bpos<length)&&(bpos<16)) {
        if(bpos>0) {
          if((bpos&1)==0) buf[outpos++]=' ';
          if((bpos&7)==0) buf[outpos++]=' ';
        }

        int val=data[start+i+bpos]&0xFF;
        buf[outpos++]=(byte)hexchars[(val>>4)&0xF];
        buf[outpos++]=(byte)hexchars[val&0xF];

        if(full) {
          buf[60+bpos]=(byte)((printmask[val]==0)?'.':printmask[val]);
        }
        bpos++;
      }

      if(full) {
        while(outpos<60) buf[outpos++]=' ';
        outpos+=bpos;
      }
      buf[outpos++]='\n';
      out.write(buf, 0, outpos);
    }
  }



  public static void bitCopy(byte src[], int srcpos,
                             byte dest[], int destpos, int length) {
    //TRACE (".bitCopy(src["+(src==null?-1:src.length)+"], srcpos="+srcpos+
    //TRACE  ", dest["+(dest==null?-1:dest.length)+"], destpos="+destpos+
    //TRACE  ", length="+length+").");

    if(((srcpos&7)==(destpos&7))&&(length>=8)) {
      if((src==dest)&&(srcpos<destpos)) { // copy last byte first
        if(((srcpos+length)&7)!=0) {
          int pos=destpos+length;
          int mask=(1<<(pos&7))-1;
          dest[pos>>3]=(byte)((dest[pos>>3]&~mask)|
                              (src[(srcpos+length)>>3]&mask));
        }
      } else { // copy first byte
        if((srcpos&7)!=0) {
          int mask=0xFF00>>(8-srcpos&7);
          dest[destpos>>3]=(byte)((dest[destpos>>3]&~mask)|
                                  (src[srcpos>>3]&mask));
        }
      }

// copy via system arraycopy (faster)
      int pos=(srcpos+7)&(~0x7);
      int bl=(length-(pos-srcpos))&(~0x7);
      if(bl>0) {
        System.arraycopy(src, pos>>3, dest, (destpos+7)>>3, bl>>3);
      }

      if((src==dest)&&(srcpos<destpos)) { // first byte
        if((srcpos&7)!=0) {
          int mask=0xFF00>>(8-srcpos&7);
          dest[destpos>>3]=(byte)((dest[destpos>>3]&~mask)|
                                  (src[srcpos>>3]&mask));
        }
      } else { // copy last byte
        if(((srcpos+length)&7)!=0) {
          pos=destpos+length;
          int mask=(1<<(pos&7))-1;
          dest[pos>>3]=(byte)((dest[pos>>3]&~mask)|
                              (src[(srcpos+length)>>3]&mask));
        }
      }
      return;
    }

// do slow bitwise copy

    if((src==dest)&&(srcpos<destpos)) {
      srcpos+=length;
      destpos+=length;

      for(int i=0; i<length; i++) {
        srcpos--;
        destpos--;
        dest[destpos>>3]=(byte)((dest[destpos>>3]&~(0x1<<(destpos&7)))|
                               (((src[srcpos>>3]>>(srcpos&7))&1)<<(destpos&7)));
      }
    } else {
      for(int i=0; i<length; i++) {
        dest[destpos>>3]=(byte)((dest[destpos>>3]&~(0x1<<(destpos&7)))|
                               (((src[srcpos>>3]>>(srcpos&7))&1)<<(destpos&7)));
        srcpos++;
        destpos++;
      }
    }
  }



/**
 *  @param val if true set the bits to 1, to 0 otherwise.
 */
  public static void bitSet(byte data[], int pos, int length, boolean val) {
    if((pos&7)!=0) { // start clearing first byte
      int i=8-(pos&7);	// number of bits in byte after pos
      int mask=0xFF00>>i;	// 1 for each bit to set
      if(length<i) {
        mask&=~(0xFF<<((pos+length)&7));
        return;
      }

      if(val) data[pos>>3]|=(byte)mask;
      else data[pos>>3]&=(byte)(~mask);
      length-=i;
      pos+=i;
    }

    if(length==0) return;
    if((length>>3)!=0) {
      java.util.Arrays.fill(data, pos>>3, (pos+length)>>3, (byte)(val?0xFF:0));
      pos+=length&(~0x7);
      length&=7;
    }

    if(length!=0) { // clear last bits
      int mask=(1<<length)-1;
      if(val) data[pos>>3]|=(byte)mask;
      else data[pos>>3]&=(byte)(~mask);
    }
  }




/** Parse the hexadecimal byte data from the inputstring.
 *  @param str hexdata string with even number of chars and
 *  no other then hexchars content.
 */
  public static byte[] parseHexString(String str)
                                      throws IllegalArgumentException {
    if((str.length()&1)!=0) throw new IllegalArgumentException();

    byte ret[]=new byte[str.length()>>1];
    for(int n=0; n<ret.length; n++) {
      int val=(Character.digit(str.charAt(n<<1),     16)<<4)|
               Character.digit(str.charAt((n<<1)+1), 16);
      if(val<0) throw new IllegalArgumentException("Illegal hex data");
      ret[n]=(byte)val;
    }

    return(ret);
  }






/** Parse the hexadecimal byte data from the inputstring.
 *  @param str hexdata string containing only hex chars and
 *  empty chars. The hex chars have always to occur in pairs. 
 */
  public static byte[] parseString(String str)
                                   throws IllegalArgumentException {

    java.io.ByteArrayOutputStream bos=new java.io.ByteArrayOutputStream();

    for(int n=0; n<str.length(); n++) {
      if(Character.isSpace(str.charAt(n))) continue;

      int val=(Character.digit(str.charAt(n),     16)<<4)|
               Character.digit(str.charAt(n+1), 16);
      if(val<0) throw new IllegalArgumentException("Illegal hex data");
      bos.write(val);

      n++;
    }


    return(bos.toByteArray());
  }


}





