package fips.util;

public class StringSplitter {


/** Split the input string into substrings. Each substring is delimited
 *  by blanks. All characters < 20 are deemed as blank.
 *  The input string may contain blanks or escaped characters
 *  between double quotes but must not contain any other
 *  characters with a keycode < 20. Currently only following
 *  characters are escaped:<Br>
 *  <Table>
 *  <Tr><Td>\t</Td><Td>tabulator</Td><Td>0x09</Td></Tr>
 *  <Tr><Td>\n</Td><Td>newline</Td><Td>0x0a</Td></Tr>
 *  <Tr><Td>\"</Td><Td>doublequote</Td><Td>0x22</Td></Tr>
 *  <Tr><Td>\\</Td><Td>backslash</Td><Td>0x5c</Td></Tr>
 *  <Tr><Td>\000</Td><Td>Character 000 dec</Td><Td>xx</Td></Tr>
 *  </Table>
 *
 *   @returns the list of substrings or null if string was empty.
 *   @throws IllegalArgumentException if quotes missing or
 *   illegal escape sequence found.
 */
  public static String[] splitString(String str) {
    //ARM (".splitString(\""+str+"\").");
    StringBuffer sbuf=new StringBuffer();

    int			pos;
    char		c;
    boolean		started;
    boolean		quoted;
    java.util.ArrayList	retl;


    retl=new java.util.ArrayList();
    started=false;
    quoted=false;

    for(pos=0; pos<str.length();) {
      while((pos<str.length())&&(str.charAt(pos)<=0x20)) pos++;

      if(pos==str.length()) break;

      c=str.charAt(pos);
      if(c=='"') { // start of quoted string
        pos++;
        if(pos==str.length()) throw new IllegalArgumentException("Closing quotes missing.");

        while(pos<str.length()) {
          c=str.charAt(pos++);
          if(c<0x20) throw new IllegalArgumentException("Illegal char between quotes.");
          if(c=='"') { // end of quotes
            if((pos!=str.length())&&(str.charAt(pos+1)>0x20)) {
              throw new IllegalArgumentException("String continues after second quote");
            }
            retl.add(sbuf.toString());
            sbuf.setLength(0);
            break;
          }

          if(pos==str.length()) throw new IllegalArgumentException("Missing closing quote");

          if(c=='\\') {
            throw new InternalError("Escape not supported");
          }

          sbuf.append(c);
        }
        continue;
      }

// normal char
      while(pos<str.length()) {
        c=str.charAt(pos++);

        if(c<=0x20) {
          break;
        }

        if(c=='"') throw new IllegalArgumentException("Illegal quote in normal string.");

        if(c=='\\') {
          throw new InternalError("No escape support yet");
        } else {
          sbuf.append(c);
        }

        if((pos==str.length())||(str.charAt(pos)<=0x20)) break;
      }

      retl.add(sbuf.toString());
      sbuf.setLength(0);
    }


    //ARM ("#: return "+retl.size()+" substrings");

    if(retl.size()==0) return(null);

    String ret[]=new String[retl.size()];
    retl.toArray(ret);
    return(ret);
  }


}








