package fips.util;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

/** This class contains an url implementation that contains
 *  the url and methods to access the data pointed out by
 *  the url.
 */

public class Url {

  String	url;

  Object	handler;


  public Url(String urlstr) {
    this.url=urlstr;

    if(urlstr.startsWith("file://")) {
      File f=new File(urlstr.substring(7));
      if((f.exists())&&(f.isDirectory())) throw new IllegalArgumentException("Directory not allowed.");
      handler=f;
    } else if(urlstr.startsWith("http://")) {
      try {
        handler=new java.net.URL(urlstr);
      } catch (Exception e) {
        throw new IllegalArgumentException("Illegal http url: "+e);
      }
    }

    if(handler==null) throw new IllegalArgumentException("Invalid url string");
  }


/** Check if the resource described by this url exists.
 */
  public boolean exists() {
    if(url.startsWith("file://")) {
      return(((File)handler).exists());
    } else if(url.startsWith("http://")) {
      return(true);
    } else {
      throw new InternalError();
    }
  }


  public boolean isWritable() {
    if(url.startsWith("file://")) {
      if(((File)handler).exists()) return(((File)handler).canWrite());

// check if directory containing f is writable
      return(true);
    } else if(url.startsWith("http://")) {
      return(false);
    } else {
      throw new InternalError();
    }
  }


  public InputStream getInputStream() throws IOException {
    if(url.startsWith("file://")) {
      if(!((File)handler).exists()) throw new IOException("File denoted by this url doesn't exist.");

      return(new java.io.FileInputStream((File)handler));
    } else if(url.startsWith("http://")) {
      return(((java.net.URL)handler).openConnection().getInputStream());
    }


    throw new InternalError();
  }



/** Truncate the resource and return an outputstream to
 *  write it again.
 */
  public OutputStream getOutputStream() throws IOException {
    if(url.startsWith("file://")) {
      if(((File)handler).exists()) ((File)handler).delete();

      return(new java.io.FileOutputStream((File)handler));
    } else if(url.startsWith("http://")) {
      throw new IOException("Httpurls are readonly");
    }

    throw new InternalError();
  }


}



